@@ -36,13 +36,19 @@ GHashTable *type_hash     = NULL;
 
 GHashTable *type_def_hash = NULL;
 
+GtkType gb_type_int;
+GtkType gb_type_long;
+GtkType gb_type_single;
+GtkType gb_type_double;
+GtkType gb_type_string;
+GtkType gb_type_boolean;
 
 GBValue *
 gb_value_new_empty (void)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_EMPTY;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_EMPTY);
 	v->v.l     = 0xcafebabe;
 
 	return v;
@@ -53,7 +59,7 @@ gb_value_new_null (void)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_NULL;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_NULL);
 	v->v.l     = 0xdeadbeef;
 
 	return v;
@@ -64,7 +70,7 @@ gb_value_new_boolean (GBBoolean b)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_BOOLEAN;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_BOOLEAN);
 	v->v.bool  = b;
 
 	return v;
@@ -75,7 +81,7 @@ gb_value_new_byte (GBByte b)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_BYTE;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_BYTE);
 	v->v.byte  = b;
 
 	return v;
@@ -86,7 +92,7 @@ gb_value_new_int (GBInt i)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_INT;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_INT);
 	v->v.i     = i;
 
 	return v;
@@ -97,7 +103,7 @@ gb_value_new_long (GBLong l)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_LONG;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_LONG);
 	v->v.l     = l;
 
 	return v;
@@ -108,7 +114,7 @@ gb_value_new_single (GBSingle f)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_SINGLE;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_SINGLE);
 	v->v.f     = f;
 
 	return v;
@@ -119,7 +125,7 @@ gb_value_new_double (GBDouble d)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_DOUBLE;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_DOUBLE);
 	v->v.d     = d;
 
 	return v;
@@ -132,7 +138,7 @@ gb_value_new_string (const GBString s)
 
 	g_return_val_if_fail (s != NULL, NULL);
 
-	v->type    = GB_VALUE_STRING;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_STRING);
 	v->v.s     = g_string_new (s->str);
 
 	return v;
@@ -145,8 +151,8 @@ gb_value_new_string_chars (const char *s)
 
 	g_return_val_if_fail (s != NULL, NULL);
 
-	v->type = GB_VALUE_STRING;
-	v->v.s  = g_string_new (s);
+	v->gtk_type = gb_gtk_type_from_value (GB_VALUE_STRING);
+	v->v.s      = g_string_new (s);
 
 	return v;
 }
@@ -159,8 +165,8 @@ gb_value_new_object (GBObject *obj)
 	g_return_val_if_fail (GB_IS_OBJECT (obj), NULL);
 
 	v = g_new (GBValue, 1);
-	v->type  = GB_VALUE_OBJECT;
-	v->v.obj = obj;
+	v->gtk_type = GTK_OBJECT (obj)->klass->type;
+	v->v.obj    = obj;
 
 	return v;
 }
@@ -170,8 +176,8 @@ gb_value_new_date (const GBDate date)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type = GB_VALUE_DATE;
-	v->v.date = date;
+	v->gtk_type = gb_gtk_type_from_value (GB_VALUE_DATE);
+	v->v.date   = date;
 
 	return v;
 }
@@ -193,8 +199,8 @@ gb_value_new_date_gdate (const GDate *date)
 	g_date_set_dmy (tmp, 1, 1, 1900);
 	val -= g_date_julian (tmp);
 
-	v->type = GB_VALUE_DATE;
-	v->v.date = val;
+	v->gtk_type = gb_gtk_type_from_value (GB_VALUE_DATE);
+	v->v.date   = val;
 	
 	g_date_free (tmp);
 
@@ -206,8 +212,8 @@ gb_value_new_time (int hr, int min, int sec)
 {
 	GBValue *v = g_new (GBValue, 1);
 	
-	v->type = GB_VALUE_DATE;
-	v->v.date = (double) ((hr * SECS_IN_HR) + (min * SECS_IN_MIN) + sec) / SECS_IN_DAY;
+	v->gtk_type = gb_gtk_type_from_value (GB_VALUE_DATE);
+	v->v.date   = (double) ((hr * SECS_IN_HR) + (min * SECS_IN_MIN) + sec) / SECS_IN_DAY;
 
 	return v;
 }
@@ -229,8 +235,8 @@ gb_value_new_date_time (const GDate *date, int hr, int min, int sec)
 	g_date_set_dmy (tmp, 1, 1, 1900);
 	val -= g_date_julian (tmp);
 
-	v->type = GB_VALUE_DATE;
-	v->v.date = val;
+	v->gtk_type = gb_gtk_type_from_value (GB_VALUE_DATE);
+	v->v.date   = val;
 	
 	/* Time is the fraction */
 	v->v.date += (double) ((hr * SECS_IN_HR) + (min * SECS_IN_MIN) + sec) / SECS_IN_DAY;
@@ -283,14 +289,14 @@ gb_value_new_frx (const char *fn_offset)
 		offset = tmp;
 
 		v                  = g_new (GBValue, 1);
-		v->type            = GB_VALUE_FRX;
+		v->gtk_type        = gb_gtk_type_from_value (GB_VALUE_FRX);
 		v->v.frx	   = g_new (GBFrx, 1);
 		v->v.frx->offset   = offset;
 		v->v.frx->filename = g_strdup (name);
 
 	} else {
 		v                  = g_new (GBValue, 1);
-		v->type            = GB_VALUE_FRX;
+		v->gtk_type        = gb_gtk_type_from_value (GB_VALUE_FRX);
 		v->v.frx	   = g_new (GBFrx, 1);
 		v->v.frx->offset   = offset;
 		v->v.frx->filename = NULL;
@@ -306,7 +312,7 @@ gb_value_new_list (GPtrArray *a)
 {
 	GBValue *v = g_new (GBValue, 1);
 
-	v->type    = GB_VALUE_LIST;
+	v->gtk_type= gb_gtk_type_from_value (GB_VALUE_LIST);
 	v->v.list  = a;
 
 	return v;
@@ -319,7 +325,7 @@ gb_value_destroy (GBValue *v)
 	if (!v)
 		return;
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_STRING:
 		g_string_free (v->v.s, TRUE);
 		break;
@@ -344,8 +350,8 @@ gb_value_copy (GBEvalContext *ec, const GBValue *v)
 
 	val = g_new (GBValue, 1);
 
-	val->type = v->type;
-	switch (v->type) {
+	val->gtk_type = v->gtk_type;
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_STRING:
 		val->v.s = g_string_new (v->v.s->str);
 		break;
@@ -363,7 +369,8 @@ gb_value_copy (GBEvalContext *ec, const GBValue *v)
 	case GB_VALUE_DECIMAL:
 	case GB_VALUE_USER_DEF:
 	case GB_VALUE_CURRENCY:
-		g_warning ("Exotic value type in copy %d", v->type);
+		g_warning ("Exotic value type in copy %d",
+			   gb_value_from_gtk_type (v->gtk_type));
 		break;
 
 	case GB_VALUE_VARIANT:
@@ -391,7 +398,7 @@ gb_value_copy_byval (GBEvalContext *ec, const GBValue *v)
 	g_return_val_if_fail (v != NULL, NULL);
 	g_return_val_if_fail (ec != NULL, NULL);
 
-	if (v->type != GB_VALUE_OBJECT)
+	if (v->gtk_type != gb_gtk_type_from_value (GB_VALUE_OBJECT))
 		return gb_value_copy (ec, v);
 	else
 		return gb_eval_exception_fire (ec, "Object copy unimplemented");
@@ -403,7 +410,7 @@ gb_value_print (FILE *sink, const GBValue *v)
 	g_return_if_fail (v != NULL);
 	g_return_if_fail (sink != NULL);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_EMPTY:
 		fprintf (sink, "Empty");
 		break;
@@ -440,7 +447,7 @@ gb_value_get_as_double (const GBValue *v)
 {
 	g_return_val_if_fail (v != NULL, 0.0);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_INT:
 		return v->v.i;
 	case GB_VALUE_LONG:
@@ -459,7 +466,8 @@ gb_value_get_as_double (const GBValue *v)
 		else
 			return 0.0;
 	default:
-		g_warning ("Unhandled type as double %d", v->type);
+		g_warning ("Unhandled type as double %d",
+			   gb_value_from_gtk_type (v->gtk_type));
 		break;
 	}
 	return 0.0;
@@ -470,7 +478,7 @@ gb_value_get_as_int (const GBValue *v)
 {
 	g_return_val_if_fail (v != NULL, 0);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_INT:
 		return (GBInt)v->v.i;
 	case GB_VALUE_LONG:
@@ -494,7 +502,7 @@ gb_value_get_as_long (const GBValue *v)
 {
 	g_return_val_if_fail (v != NULL, 0);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_INT:
 		return (GBLong)v->v.i;
 	case GB_VALUE_LONG:
@@ -519,7 +527,7 @@ gb_value_get_as_string (const GBValue *v)
 
 	g_return_val_if_fail (v != NULL, g_string_new (""));
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 
 	case GB_VALUE_EMPTY:
 	case GB_VALUE_NULL:
@@ -575,7 +583,8 @@ gb_value_get_as_string (const GBValue *v)
 	default:
 		g_warning ("Unhandled value as string");
 		str = g_string_new ("");
-		g_string_sprintf (str, "error value 0x%x", v->type);
+		g_string_sprintf (str, "error value 0x%x",
+				  gb_value_from_gtk_type (v->gtk_type));
 	}
 	return str;
 }
@@ -597,7 +606,7 @@ gb_value_get_as_boolean (const GBValue* v)
 {
 	g_return_val_if_fail (v != NULL, GBFalse);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 
 	case GB_VALUE_EMPTY :
 		return GBFalse;
@@ -657,7 +666,7 @@ gb_value_get_as_date (const GBValue *v)
 	
 	g_return_val_if_fail (v != NULL, d);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 
 	case GB_VALUE_DATE:
 		d = v->v.date;
@@ -684,7 +693,7 @@ gb_value_get_as_gdate (const GBValue *v)
 
 	g_return_val_if_fail (v != NULL, d);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 		
 	case GB_VALUE_DATE:
 		g_date_set_dmy (tmp, 1, 1, 100);
@@ -710,8 +719,7 @@ gb_value_time_get_hr (const GBValue *v)
 
 	tmp = modf (v->v.date, &i);
 
-	return (GBInt) floor(floor (tmp * SECS_IN_DAY) / SECS_IN_HR);  
-
+	return (GBInt) floor (floor (tmp * SECS_IN_DAY) / SECS_IN_HR);  
 }
 
 GBInt
@@ -747,8 +755,8 @@ gb_value_time_get_sec (const GBValue *v)
  */
 GBValue *
 gb_value_promote (GBEvalContext *ec,
-		  const GBValue *v,
-		  GBValueType    to)
+		  GtkType        to,
+		  const GBValue *v)
 {
 	GBValue    *ret = NULL;
 	GBValue    *nv;
@@ -757,12 +765,13 @@ gb_value_promote (GBEvalContext *ec,
 	g_return_val_if_fail (v != NULL, NULL);
 	g_return_val_if_fail (ec != NULL, NULL);
 
-	if (v->type == to || to == GB_VALUE_VARIANT)
+	if (v->gtk_type == to || to == gb_gtk_type_from_value (GB_VALUE_VARIANT) ||
+	    gtk_type_is_a (v->gtk_type, to))
 		return gb_value_copy (ec, v);
 
-	from = v->type;
+	from = gb_value_from_gtk_type (v->gtk_type);
 
-	switch (to) {
+	switch (gb_value_from_gtk_type (to)) {
 	case GB_VALUE_BOOLEAN:
 		switch (from) {
 			/*
@@ -777,7 +786,7 @@ gb_value_promote (GBEvalContext *ec,
 			break;
 
 		case GB_VALUE_INTEGER:
-		        nv = gb_value_promote (ec, v, GB_VALUE_LONG);
+		        nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_LONG), v);
 			if (nv && nv->v.l != 0)
 				ret = gb_value_new_boolean (TRUE);
 			else
@@ -786,7 +795,7 @@ gb_value_promote (GBEvalContext *ec,
 			break;
 			
 		case GB_VALUE_FLOAT:
-		        nv = gb_value_promote (ec, v, GB_VALUE_DOUBLE);
+		        nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_DOUBLE), v);
 			if (nv && nv->v.d != 0.0)
 				ret = gb_value_new_boolean (TRUE);
 			else
@@ -806,7 +815,7 @@ gb_value_promote (GBEvalContext *ec,
 
 	/* Chain up then check */
 	case GB_VALUE_BYTE:
-		nv = gb_value_promote (ec, v, GB_VALUE_LONG);
+		nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_LONG), v);
 		if (nv &&
 		    nv->v.l <= GB_VALUE_BYTE_MAX &&
 		    nv->v.l >= GB_VALUE_BYTE_MIN)
@@ -815,7 +824,7 @@ gb_value_promote (GBEvalContext *ec,
 		break;
 
 	case GB_VALUE_INT:
-		nv = gb_value_promote (ec, v, GB_VALUE_LONG);
+		nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_LONG), v);
 		if (nv &&
 		    nv->v.l <= GB_VALUE_INT_MAX &&
 		    nv->v.l >= GB_VALUE_INT_MIN)
@@ -878,7 +887,7 @@ gb_value_promote (GBEvalContext *ec,
 		break;
 
 	case GB_VALUE_SINGLE:
-		nv = gb_value_promote (ec, v, GB_VALUE_DOUBLE);
+		nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_DOUBLE), v);
 		if (nv)
 /*
  * FIXME: Need some sensible bounds, or to know if we just loose precision.
@@ -946,14 +955,9 @@ gb_value_promote (GBEvalContext *ec,
 			break;
 
 		case GB_VALUE_BYTE:
-			nv = gb_value_promote (ec, v, GB_VALUE_LONG);
-			ret = gb_value_promote (ec, nv, GB_VALUE_STRING);
-			gb_value_destroy (nv);
-			break;
-
 		case GB_VALUE_INT:
-			nv = gb_value_promote (ec, v, GB_VALUE_LONG);
-			ret = gb_value_promote (ec, nv, GB_VALUE_STRING);
+			nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_LONG), v);
+			ret = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_STRING), nv);
 			gb_value_destroy (nv);
 			break;
 
@@ -968,8 +972,8 @@ gb_value_promote (GBEvalContext *ec,
 		}
 
 		case GB_VALUE_SINGLE:
-			nv = gb_value_promote (ec, v, GB_VALUE_DOUBLE);
-			ret = gb_value_promote (ec, nv, GB_VALUE_STRING);
+			nv = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_DOUBLE), v);
+			ret = gb_value_promote (ec, gb_gtk_type_from_value (GB_VALUE_STRING), nv);
 			gb_value_destroy (nv);
 			break;
 
@@ -1022,8 +1026,7 @@ gb_value_promote (GBEvalContext *ec,
 		
 		case GB_VALUE_FRX:
 		{
-			GPtrArray	*textptr;
-
+			GPtrArray *textptr;
 
 			textptr = gb_get_frx_text (v);
 
@@ -1088,8 +1091,8 @@ gb_value_promote (GBEvalContext *ec,
 		return ret;
 
 	return gb_eval_exception_firev (ec, "(gb_value_promote) Can't promote %s to %s",
-					gb_value_type_name (from),
-					gb_value_type_name (to));
+					gtk_type_name (gb_gtk_type_from_value (from)),
+					gtk_type_name (to));
 }
 
 gboolean
@@ -1097,7 +1100,7 @@ gb_value_is_integer (const GBValue *v)
 {
 	g_return_val_if_fail (v != NULL, FALSE);
 
-	switch (v->type) {
+	switch (gb_value_from_gtk_type (v->gtk_type)) {
 	case GB_VALUE_INTEGER:
 		return TRUE;
 	default:
@@ -1106,7 +1109,7 @@ gb_value_is_integer (const GBValue *v)
 }
 
 GBValue *
-gb_value_new_default (GBEvalContext *ec, GBValueType t)
+gb_value_new_default (GBEvalContext *ec, GtkType t)
 {
 	GBValue *val;
 
@@ -1114,25 +1117,27 @@ gb_value_new_default (GBEvalContext *ec, GBValueType t)
 
 	val = g_hash_table_lookup (type_def_hash, GINT_TO_POINTER (t));
 	if (!val)
-		return gb_eval_exception_firev (ec, "Uknown type code %d", t);
+		return NULL;
 
 	return gb_value_copy (ec, val);
 }
 
-GBValueType
-gb_value_type_from_name (const char *type_name)
+GtkType
+gb_gtk_type_from_name (const char *vb_type_name)
 {
-	GBValueType t;
+	GtkType t;
+	char   *name;
 
-	g_return_val_if_fail (type_hash != NULL, GB_VALUE_EMPTY);
-	g_return_val_if_fail (type_name != NULL, GB_VALUE_EMPTY);
+	g_return_val_if_fail (vb_type_name != NULL, 0);
 
-	t = GPOINTER_TO_INT (g_hash_table_lookup (type_hash, type_name));
+	if (!strcmp (vb_type_name, "variant"))
+		return 0;
 
-/*	if (t == GB_VALUE_EMPTY)
-	FIXME: do we need a more robust condition here ?
-	is it possible to declare variables of type Empty ?
-	g_warning ("'%s' maps to an empty type", type_name);*/
+	name = gb_gtk_type_name (vb_type_name);
+
+	t = gtk_type_from_name (name);
+
+	g_free (name);
 
 	return t;
 }
@@ -1151,30 +1156,12 @@ find_name_cb (gpointer key, gpointer value, gpointer user_data)
 		fct->name = key;
 }
 
-const char *
-gb_value_type_name (GBValueType t)
-{
-	find_closure_t fct;
-
-	fct.t    = t;
-	fct.name = NULL;
-
-	if (t == GB_VALUE_OBJECT)
-		return "Object";
-
-	g_hash_table_foreach (type_hash, find_name_cb, &fct);
-	if (!fct.name)
-		g_warning ("No type name for %d", t);
-
-	return fct.name;
-}
-
 static void
 add_type_map (const char *name, GBValue *val)
 {
 	g_hash_table_insert (type_hash, g_strdup (name),
-			     GINT_TO_POINTER (val->type));
-	g_hash_table_insert (type_def_hash, GINT_TO_POINTER (val->type), val);
+			     GINT_TO_POINTER (val->gtk_type));
+	g_hash_table_insert (type_def_hash, GINT_TO_POINTER (val->gtk_type), val);
 }
 
 gint
@@ -1202,14 +1189,159 @@ g_strcase_hash (gconstpointer v)
 	return h /* % M */;
 }
 
+typedef struct {
+	const char *name;
+	GtkType     created_type;
+} GBBuiltinType;
+
+static GBBuiltinType
+builtin_types [] = {
+	{ "gb-empty" },		/* 0 */
+	{ "gb-null" },		/* 1 */
+	{ "gb-integer" },	/* 2 */
+	{ "gb-long" },		/* 3 */
+	{ "gb-single" },	/* 4 */
+	{ "gb-double" },	/* 5 */
+	{ "gb-currency" },	/* 6 */
+	{ "gb-date" },		/* 7 */
+	{ "gb-string" },	/* 8 */
+	{ "gb-object" },	/* 9 */
+	{ "gb-error" },		/* 10 */
+	{ "gb-boolean" },	/* 11 */
+	{ "gb-variant" },	/* 12 */
+	{ "gb-dataobject" },	/* 13 */
+	{ "gb-decimal" },	/* 14 */
+	{ NULL },		/* 15 */
+	{ NULL },		/* 16 */
+	{ "gb-byte" },	        /* 17 */
+	{ NULL },		/* 18 */
+	{ NULL },		/* 19 */
+	{ NULL },		/* 20 */
+	{ NULL },		/* 21 */
+	{ NULL },		/* 22 */
+	{ NULL },		/* 23 */
+	{ NULL },		/* 24 */
+	{ NULL },		/* 25 */
+	{ NULL },		/* 26 */
+	{ NULL },		/* 27 */
+	{ NULL },		/* 28 */
+	{ NULL },		/* 29 */
+	{ NULL },		/* 30 */
+	{ NULL },		/* 31 */
+	{ NULL },		/* 32 */
+	{ NULL },		/* 33 */
+	{ NULL },		/* 34 */
+	{ NULL },		/* 35 */
+	{ "gb-userdef" },	/* 36 */
+	{ "gb-frx" },		/* 37 */
+	{ "gb-list" },		/* 38 */
+};
+
+GtkType
+gb_fundamental_type ()
+{
+	static int type = 0;
+	
+	if (!type) {
+		GtkTypeInfo info = { "gb-fundamental-type" };
+		type = gtk_type_unique (0, &info);
+	}
+
+	return type;
+}
+
+GtkType
+gb_gtk_type_from_value (GBValueType t)
+{
+	if (t == GB_VALUE_VARIANT)
+		return 0;
+	else if (t >= 0 && t < sizeof (builtin_types) / sizeof (GBBuiltinType))
+		return builtin_types [t].created_type;
+	else
+		return builtin_types [GB_TYPE_OBJECT].created_type;
+}
+
+GBValueType
+gb_value_from_gtk_type (GtkType t)
+{
+	int i;
+
+	for (i = 0; i < sizeof (builtin_types) / sizeof (GBBuiltinType); i++) {
+		if (builtin_types [i].created_type == t)
+			return i;
+	}
+
+	return GB_TYPE_OBJECT;
+}
+
+char *
+gb_gtk_type_name (const char *gb_name)
+{
+	char *name;
+
+	g_return_val_if_fail (gb_name != NULL, NULL);
+
+	name = g_strconcat ("gb-", gb_name, NULL);
+
+	g_strdown (name);
+
+	return name;
+}
+
+char *
+gb_type_name_from_gtk (const char *gtk_name)
+{
+	if (gtk_name) {
+		if (strncmp (gtk_name, "gb-", 3))
+			return g_strdup (gtk_name + 3);
+		else
+			return g_strdup (gtk_name);
+	} else
+		return NULL;
+}
+
 void
 gb_value_init (void)
 {
 	GDate *date;
+	int    i;
+	GtkTypeInfo info;
 
+	/* Register Gtk Types */
+	for (i = 0; i < sizeof (builtin_types) / sizeof (GBBuiltinType); i++) {
+		info.object_size = 0;
+		info.class_size  = 8;
+		info.class_init_func  = NULL;
+		info.object_init_func = NULL;
+		info.reserved_1 = NULL;
+		info.reserved_2 = NULL;
+		info.base_class_init_func = NULL;
+
+		if ((info.type_name = (char *)builtin_types [i].name)) {
+			builtin_types [i].created_type =
+				gtk_type_unique (gb_fundamental_type (), &info);
+			
+			if (!builtin_types [i].created_type)
+				g_warning ("Failed to create type '%s'", info.type_name);
+		}
+	}
+
+	gb_type_int     = builtin_types [GB_VALUE_INT].created_type;
+	gb_type_long    = builtin_types [GB_VALUE_LONG].created_type;
+	gb_type_single  = builtin_types [GB_VALUE_SINGLE].created_type;
+	gb_type_double  = builtin_types [GB_VALUE_DOUBLE].created_type;
+	gb_type_string  = builtin_types [GB_VALUE_STRING].created_type;
+	gb_type_boolean = builtin_types [GB_VALUE_BOOLEAN].created_type;
+
+	builtin_types [GB_VALUE_OBJECT].created_type =
+		gb_object_get_type ();
+	builtin_types [GB_VALUE_VARIANT].created_type = 0;
+
+	/* Register various internal types */
 	GBTrue_value  = gb_value_new_boolean (GBTrue);
 	GBFalse_value = gb_value_new_boolean (GBFalse);
 
+	/* Register type name mappings */
 	type_hash     = g_hash_table_new (g_strcase_hash, g_strcase_equal);
 	type_def_hash = g_hash_table_new (NULL, NULL); /* hear it type */
 
@@ -1251,6 +1383,7 @@ kill_type_def_map (gpointer key, gpointer value, gpointer user_data)
 	if (value)
 		gb_value_destroy (value);
 }
+
 void
 gb_value_shutdown (void)
 {
