@@ -11,8 +11,11 @@
 
 #include <gb/gb.h>
 
-/* Values */
 
+#define GB_FUNDAMENTAL_TYPE  (gb_fundamental_type ())
+#define GB_IS_A_FUNDAMENTAL(t) (gtk_type_is_a (t, GB_FUNDAMENTAL_TYPE))
+
+/* Values */
 #define GBTrue               TRUE
 #define GBFalse              FALSE
 
@@ -68,6 +71,13 @@ typedef enum {
 	GB_VALUE_TYPE_MAX    = 39
 } GBValueType;
 
+GtkType gb_type_int;
+GtkType gb_type_long;
+GtkType gb_type_single;
+GtkType gb_type_double;
+GtkType gb_type_string;
+GtkType gb_type_boolean;
+
 #define GB_VALUE_IS_EMPTY(v)    (!(v)->type)
 #define GB_VALUE_INTEGER             GB_VALUE_LONG: \
 				case GB_VALUE_INT:  \
@@ -82,7 +92,7 @@ typedef enum {
 
 /* See: vagrpDataType.htm */
 struct _GBValue {
-	GBValueType type;
+	GtkType gtk_type;
 	union {
 		GBByte              byte;
 		GBBoolean           bool;
@@ -122,12 +132,17 @@ GBValue *gb_value_new_date         (const GBDate date);
 GBValue *gb_value_new_date_gdate   (const GDate *date);
 GBValue *gb_value_new_time         (int hr, int min, int sec);
 GBValue *gb_value_new_date_time    (const GDate *date, int hr, int min, int sec);
-GBValue *gb_value_new_default      (GBEvalContext *ec, GBValueType type);
+GBValue *gb_value_new_default      (GBEvalContext *ec, GtkType type);
 GBValue *gb_value_new_frx          (const char *s);
 
 
-const char    *gb_value_type_name        (GBValueType t);
-GBValueType    gb_value_type_from_name   (const char *type_name);
+GtkType        gb_fundamental_type       (void);
+GtkType        gb_gtk_type_from_value    (GBValueType t);
+GBValueType    gb_value_from_gtk_type    (GtkType     t);
+char          *gb_gtk_type_name          (const char *gb_name);
+char          *gb_type_name_from_gtk     (const char *gtk_name);
+GtkType        gb_gtk_type_from_name     (const char *vb_type_name);
+
 /* Destructor */
 void           gb_value_destroy          (GBValue *);
 
@@ -149,8 +164,8 @@ gboolean       gb_value_is_integer       (const GBValue *);
 
 /* Promotions */
 GBValue       *gb_value_promote         (GBEvalContext  *ec,
-					 const GBValue  *v,
-					 GBValueType     to);
+					 GtkType         to,
+					 const GBValue  *v);
 
 /* Private */
 void           gb_value_init             (void);
