@@ -40,14 +40,6 @@ typedef struct _GBEvalContextClass  GBEvalContextClass;
 typedef struct _GBObject            GBObject;
 typedef struct _GBObjectClass       GBObjectClass;
 
-typedef struct _GBObjectPriv        GBObjectPriv;
-typedef struct _GBObjectPrivClass   GBObjectPrivClass;
-
-typedef void  (GBObjectCopy)       (GBEvalContext *ec,
-				    GBObject      *copy_obj,
-				    GBObject      *templ_obj);
-typedef void  (GBObjectDestructor) (GBObject      *object);
-
 /* gb-expr.h */
 typedef struct _GBExpr     GBExpr;
 typedef GSList             GBExprList;
