@@ -50,7 +50,7 @@ gbrun_eval_context_new (const char        *module_name,
 
 	ec = gtk_type_new (GBRUN_TYPE_EVAL_CONTEXT);
 
-	ec->stack = gbrun_stack_new ();
+	ec->stack = gtk_type_new (GBRUN_TYPE_STACK);
 	gbrun_stack_call (ec, module_name);
 	ec->flags = flags;
 
@@ -75,7 +75,7 @@ gbrun_eval_context_destroy (GtkObject *o)
 
 	ec = GBRUN_EVAL_CONTEXT (o);
 	if (ec) {
-		gbrun_stack_destroy (ec->stack);
+		gtk_object_unref (GTK_OBJECT (ec->stack));
 		ec->stack = NULL;
 		gbrun_files_clean (ec);
 	}
@@ -141,22 +141,22 @@ gbrun_exception_firev (GBRunEvalContext *ec,
 	return NULL;
 }
 
-GBRunObject *
+GBObject *
 gbrun_eval_context_me_get (GBRunEvalContext *ec)
 {
 	g_return_val_if_fail (ec != NULL, NULL);
 
 	if (!ec->me)
 		return NULL;
-	
-	g_return_val_if_fail (ec->me->type == GB_VALUE_OBJECT, NULL);
 
-	return ec->me->v.obj;
+	g_return_val_if_fail (GB_IS_AN_OBJECT (ec->me->gtk_type), NULL);
+
+	return GB_OBJECT (ec->me->v.obj);
 }
 
 void
 gbrun_eval_context_me_set (GBRunEvalContext *ec,
-			   GBRunObject      *obj)
+			   GBObject         *obj)
 {
 	g_return_if_fail (ec != NULL);
 
@@ -265,7 +265,6 @@ GBValue *
 gbrun_eval_as (GBRunEvalContext  *ec,
 	       const GBExpr      *expr,
 	       GBValueType        ret_type)
-
 {
 	GBValue *tmp, *ans;
 
@@ -273,7 +272,8 @@ gbrun_eval_as (GBRunEvalContext  *ec,
 	if (!tmp)
 		return NULL;
 
-	ans = gb_value_promote (GB_EVAL_CONTEXT (ec), tmp, ret_type);
+	ans = gb_value_promote (GB_EVAL_CONTEXT (ec),
+				gb_gtk_type_from_value (ret_type), tmp);
 	gb_value_destroy (tmp);
 
 	return ans;
