@@ -55,9 +55,9 @@ GBValue          *gbrun_eval_as                (GBRunEvalContext  *ec,
 						const GBExpr      *expr,
 						GBValueType        ret_type);
 
-GBRunObject      *gbrun_eval_context_me_get    (GBRunEvalContext *ec);
+GBObject         *gbrun_eval_context_me_get    (GBRunEvalContext *ec);
 void              gbrun_eval_context_me_set    (GBRunEvalContext *ec,
-						GBRunObject      *obj);
+						GBObject         *obj);
 
 GBRunProject     *gbrun_eval_context_proj_get  (GBRunEvalContext *ec);
 void              gbrun_eval_context_proj_push (GBRunEvalContext *ec,
@@ -72,4 +72,6 @@ GBValue          *gbrun_exception_fire         (GBRunEvalContext *ec,
 GBValue          *gbrun_exception_firev        (GBRunEvalContext *ec,
 						const char *format, ...);
 
+
+
 #endif /* GBRUN_EVAL_H */
