@@ -30,6 +30,13 @@
 		return FALSE; \
 	}
 
+#define FILE_FUNC_SEC_CHECK(ec,txt) \
+	if (ec->flags & GBRUN_SEC_IO) { \
+		gbrun_exception_firev (ec, "Insufficient privilege to %s file", \
+				       txt); \
+		return FALSE; \
+	}
+
 struct _GBRunFileHandle {
         /* VB related */
         int       used;
@@ -72,11 +79,13 @@ internal_handle_from_gb_no (GBRunEvalContext *ec, int fileno)
 
 GBValue *
 gbrun_func_eof (GBRunEvalContext *ec,
-		GBObject         *object,
+		GBRunObject      *object,
 		GBValue         **args)
 {
 	GBRunFileHandle *h;
 
+	FILE_FUNC_SEC_CHECK (ec, "eof");
+
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
@@ -92,7 +101,7 @@ gbrun_func_eof (GBRunEvalContext *ec,
 
 GBValue *
 gbrun_func_freefile (GBRunEvalContext *ec,
-		     GBObject         *object,
+		     GBRunObject      *object,
 		     GBValue         **args)
 {
 	int              max  = -1;
@@ -100,11 +109,12 @@ gbrun_func_freefile (GBRunEvalContext *ec,
 	int              loop = 0;
 	GBRunFileHandle *handles = ec->file_handles;
 
-	if (args [0]) {
+	FILE_FUNC_SEC_CHECK (ec, "freefile");
+
+	if (args [0])
 		arg = args[0]->v.i;
-	} else {
+	else
 		arg = 0;
-	}
 
 	switch (arg) {
 
@@ -145,13 +155,15 @@ gbrun_func_freefile (GBRunEvalContext *ec,
 
 GBValue *
 gbrun_func_loc (GBRunEvalContext *ec,
-		GBObject         *object,
+		GBRunObject      *object,
 		GBValue         **args)
 {
 	GBRunFileHandle *h;
 	GBValue         *res;
 	long             pos;
 
+	FILE_FUNC_SEC_CHECK (ec, "loc");
+
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
@@ -175,18 +187,15 @@ gbrun_func_loc (GBRunEvalContext *ec,
 long
 get_file_len (FILE *fin)
 {
-	long  len;
-	long  pos;
+	long len, pos;
 
 	pos = ftell (fin);
-	if (fseek (fin, 0, SEEK_END)) {
+	if (fseek (fin, 0, SEEK_END))
 		return -1;
-	}
 
 	len = ftell (fin);
-	if (fseek (fin, pos, SEEK_SET)) {
+	if (fseek (fin, pos, SEEK_SET))
 		return -1;
-	}
 
 	return len;
 }
@@ -194,13 +203,15 @@ get_file_len (FILE *fin)
 
 GBValue *
 gbrun_func_lof (GBRunEvalContext *ec,
-		GBObject         *object,
+		GBRunObject      *object,
 		GBValue         **args)
 {
 	GBRunFileHandle *h;
 	GBValue         *res;
 	long             len;
 
+	FILE_FUNC_SEC_CHECK (ec, "lof");
+
 	GB_IS_VALUE (ec, args[0], GB_VALUE_INT);
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
 
@@ -219,13 +230,15 @@ gbrun_func_lof (GBRunEvalContext *ec,
 
 GBValue *
 gbrun_func_seek (GBRunEvalContext *ec,
-		 GBObject         *object,
+		 GBRunObject      *object,
 		 GBValue         **args)
 {
 	GBRunFileHandle *h;
 	GBValue         *res;
 	long             pos;
 
+	FILE_FUNC_SEC_CHECK (ec, "seek");
+
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
@@ -241,23 +254,6 @@ gbrun_func_seek (GBRunEvalContext *ec,
 	return res;
 }
 
-gboolean
-gbrun_files_clean (GBRunEvalContext *ec)
-{
-	int              loop;
-	GBRunFileHandle *handles = ec->file_handles;
-
-	for (loop = 0; loop < 512; loop++) {
-		if (handles [loop].used == TRUE) {
-			fclose (handles [loop].file);
-		}
- 	}
-
-	g_free (ec->file_handles);
-
-	return TRUE;
-}
-
 gboolean
 gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 {
@@ -273,8 +269,8 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	FILE_STMT_SEC_CHECK (ec, "open");
 
-	if (filename->type != GB_VALUE_STRING &&
-	    handle->type   != GB_VALUE_INT) {
+	if (filename->gtk_type != gb_gtk_type_from_value (GB_VALUE_STRING) &&
+	    handle->gtk_type   != gb_gtk_type_from_value (GB_VALUE_INT)) {
 		gbrun_exception_fire (ec, "Filename and filenumber are of incorrect types");
 		return FALSE;
 	}
@@ -315,7 +311,7 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 		if (lock != 1) {
 			fd = fopen (filename->v.s->str, "a");
 		} else {
-			fd - fdopen (filedes, "a");
+			fd = fdopen (filedes, "a");
 		}
 		break;
 
@@ -348,7 +344,7 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	h->fileno = handle->v.i;
 	h->mode   = stmt->parm.open.mode;
-    h->file   = fd;
+	h->file   = fd;
 	h->used   = TRUE;
 
 	if (stmt->parm.open.recordlen) {
@@ -383,8 +379,7 @@ gbrun_stmt_close (GBRunEvalContext *ec, const GBStatement *stmt)
 		GBExpr          *handle = l->data;
 		GBValue         *fileno;
 
-		fileno = gbrun_eval_as (ec, handle,
-					GB_VALUE_INT);
+		fileno = gbrun_eval_as (ec, handle, GB_VALUE_INT);
 
 		h = internal_handle_from_gb_no (ec, fileno->v.i);
 
@@ -449,8 +444,7 @@ gbrun_stmt_line_input (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	FILE_STMT_SEC_CHECK (ec, "line input from");
 
-	handle = gbrun_eval_as (ec, stmt->parm.line_input.handle,
-				GB_VALUE_INT);
+	handle = gbrun_eval_as (ec, stmt->parm.line_input.handle, GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, handle->v.i);
 	gb_value_destroy (handle);
@@ -458,6 +452,7 @@ gbrun_stmt_line_input (GBRunEvalContext *ec, const GBStatement *stmt)
 	if (objref->type != GB_EXPR_OBJREF) {
 		gbrun_exception_fire (ec, "Need an object ref");
 		return FALSE;
+
 	} else if (h->mode != GB_OPEN_INPUT  &&
 		   h->mode != GB_OPEN_RANDOM &&
 		   h->mode != GB_OPEN_BINARY) {
@@ -637,7 +632,8 @@ gbrun_stmt_put (GBRunEvalContext *ec, const GBStatement *stmt)
 		fseek (h->file, (h->recordlen) * (recordnum->v.l - 1), SEEK_SET);
 
         tmp = gbrun_eval_objref (ec, stmt->parm.put.objref);
-	str = gb_value_promote (GB_EVAL_CONTEXT (ec), tmp, GB_VALUE_STRING);
+	str = gb_value_promote (GB_EVAL_CONTEXT (ec),
+				gb_gtk_type_from_value (GB_VALUE_STRING), tmp);
 	gb_value_destroy (tmp);
 
 	if ((fputs (str->v.s->str, h->file)) == EOF) {
@@ -694,5 +690,20 @@ void
 gbrun_files_init (GBRunEvalContext *ec)
 {
 	ec->file_handles = g_new0 (GBRunFileHandle, 512);
+}
 
+gboolean
+gbrun_files_clean (GBRunEvalContext *ec)
+{
+	int              loop;
+	GBRunFileHandle *handles = ec->file_handles;
+
+	for (loop = 0; loop < 512; loop++) {
+		if (handles [loop].used == TRUE)
+			fclose (handles [loop].file);
+ 	}
+
+	g_free (ec->file_handles);
+
+	return TRUE;
 }
