@@ -1,4 +1,3 @@
-
 /*
  * GNOME Basic Object runtime ...
  *
@@ -7,6 +6,7 @@
  *
  * Copyright 2000, Helix Code Inc.
  */
+
 #include <math.h>
 #include <setjmp.h>
 #include <ctype.h>
@@ -19,160 +19,54 @@
 #include <gbrun/gbrun-value.h>
 #include <gbrun/gbrun-statement.h>
 #include <gbrun/gbrun-project.h>
+#include <gbrun/gbrun-global.h>
 
 #undef VB_METHOD_DEBUG
+#undef VB_INIT_DEBUG
 
-#define PRIV(o)        ((GBRunObjectPriv *)gb_object_get_priv ((o), gbrun_object_class ()))
+struct _GBRunObjectPrivate {
+	GBRunStackLevel    *vars;
+};
 
-GHashTable     *gbrun_object_hash = NULL;
-GList          *gbrun_global_objects = NULL;
+struct _GBRunObjectPrivateClass {
+	GSList *variables;
 
-GtkObjectClass *gbrun_object_parent = NULL;
+	GSList *classes;
 
-static void gbrun_object_add_variables (GBRunEvalContext *ec,
-					GBRunObject      *obj,
-					GHashTable       *vars);
-GBRunObjectClass *
-gbrun_object_lookup (const char *name)
+	GSList *properties;
+
+	GSList *methods;
+};
+
+static void
+gbrun_object_var_add (GBRunEvalContext *ec,
+		      GBRunObject      *obj,
+		      const char       *name,
+		      GBValue          *value)
 {
-	GBRunObjectClass *tmp;
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (value != NULL);
+	g_return_if_fail (GBRUN_IS_OBJECT (obj));
 
-	g_return_val_if_fail (name != NULL, FALSE);
-
-	tmp = (GBRunObjectClass *) g_hash_table_lookup (gbrun_object_hash,
-							(gconstpointer) name);
-
-	return tmp;
-}
-
-GBRunObject *
-gbrun_object_new (GBRunEvalContext *ec,
-		  const char       *vb_name)
-{
-	GBRunObjectClass     *klass;
-	GBRunObjectPriv      *priv;
-	GBRunObjectPrivClass *pc;
-	GBObject             *obj;
-
-	klass = gbrun_object_lookup (vb_name);
-	
-	obj = gb_object_new (GB_EVAL_CONTEXT (ec),
-			     klass);
-
-	priv = PRIV (obj);
-	if (priv) { /* Builtin objects don't inherit from GBRunObject */
-		/*
-		 * FIXME: some idiot could stick vars on all other class' we only
-		 * get the GBRunObject's
-		 */
-		pc = GBRUN_OBJECT_PRIV_GET_CLASS (priv);
-		if (pc->vars)
-			gbrun_object_add_variables (ec, obj, pc->vars);
-	}
-
-	return obj;
-}
-
-void
-gbrun_object_priv_copy (GBEvalContext *ec,
-			GBObject      *copy,
-			GBObject      *templ)
-{
-	GBRunObjectPriv *priv = PRIV (copy);
-
-	if (templ == NULL) {
-		priv->vars = NULL;
-	} else if (priv->vars)
-		g_warning ("FIXME: we need to copy vars.");
-}
-
-void
-gbrun_object_priv_destruct (GBObject *object)
-{
-	GBRunObjectPriv *priv = PRIV (object);
-
-	if (priv->vars) {
-		gbrun_stack_level_destroy (priv->vars);
-		priv->vars = NULL;
-	}
-}
-
-/**
- * gbrun_object_register:
- * 
- * Return value: the newly created class object or NULL on failure.
- **/
-void
-gbrun_object_register (const char *vb_name, GBObjectClass *klass)
-{
-	GBRunObjectPrivClass *pc;
-
-	g_return_if_fail (klass != NULL);
-
-	if (!vb_name) /* Internal object */
-		return;
-
-	/* Setup name */
-	pc = gbrun_object_class_get_priv (klass);
-	if (pc->name)
-		printf ("Registering '%s' over '%s'\n", vb_name, pc->name);
-	else if (vb_name)
-		pc->name = g_strdup (vb_name);
-	
-	if (g_hash_table_lookup (gbrun_object_hash, vb_name))
-		g_warning ("Object '%s' already registered", vb_name);
-	else
-		g_hash_table_insert (gbrun_object_hash, (gpointer)vb_name, klass);
-}
-
-GBObjectClass *
-gbrun_object_class (void)
-{
-	static GBObjectClass *oc = NULL;
-
-	if (!oc) {
-		static GBRunObjectPrivClass p;
-		gb_object_priv_class_init (&p.priv, "--GBRun.Object--",
-					   sizeof (GBRunObjectPriv),
-					   gbrun_object_priv_copy,
-					   gbrun_object_priv_destruct);
-		gbrun_object_priv_class_init (&p);
-
-		oc = gb_object_class_new_single (&p.priv, NULL);
-	}
-		
-	return oc;
+	gbrun_stack_level_add (ec, obj->priv->vars, name, value);
 }
 
 static void
-make_object_vars (GBRunObjectPriv *priv)
-{
-	g_return_if_fail (priv != NULL);
-
-	if (!priv->vars)
-		/* FIXME: a more descriptive stack name would be good */
-		priv->vars = gbrun_stack_level_new ("Object vars");
-}
-
-void
 gbrun_object_var_set (GBRunEvalContext *ec,
 		      GBRunObject      *obj,
 		      const char       *name,
 		      GBValue          *value)
 {
-	GBValue        **val;
-	GBRunObjectPriv *priv;
+	GBValue **val;
 
-	g_return_if_fail (obj != NULL);
-	priv = PRIV (obj);
-	g_return_if_fail (priv != NULL);
+	g_return_if_fail (GBRUN_IS_OBJECT (obj));
 
-	make_object_vars (priv);
+	val = gbrun_stack_level_lookup (obj->priv->vars, name);
 
-	val = gbrun_stack_level_lookup (priv->vars, name);
-	if (!val) {
+	if (!val)
 		gbrun_object_var_add (ec, obj, name, value);
-	} else {
+
+	else {
 		if (*val)
 			gb_value_destroy (*val);
 
@@ -180,42 +74,45 @@ gbrun_object_var_set (GBRunEvalContext *ec,
 	}
 }
 
-GBValue **
+static GBValue *
 gbrun_object_var_get (GBRunEvalContext *ec,
 		      GBRunObject      *obj,
 		      const char       *name)
 {
-	GBValue        **val;
-	GBRunObjectPriv *priv;
+	GBValue  *ret;
+	GBValue **val;
 
-	g_return_val_if_fail (obj != NULL, NULL);
-	priv = PRIV (obj);
-	g_return_val_if_fail (priv != NULL, NULL);
+	g_return_val_if_fail (GBRUN_IS_OBJECT (obj), NULL);
 
-	make_object_vars (priv);
+	val = gbrun_stack_level_lookup (obj->priv->vars, name);
 
-	val = gbrun_stack_level_lookup (priv->vars, name);
+	ret = (val && *val) ?
+		gb_value_copy (GB_EVAL_CONTEXT (ec), *val) : NULL;
 
-	return val;
+/*	g_warning ("Var get of '%s' on '%s' '%p'",
+	name, gtk_type_name (GTK_OBJECT (obj)->klass->type), ret);*/
+	
+	return ret;
 }
 
-void
-gbrun_object_var_add (GBRunEvalContext *ec,
-		      GBRunObject      *obj,
-		      const char       *name,
-		      GBValue          *value)
+static gboolean
+gbrun_object_set_arg_unimplemented (GBRunEvalContext  *ec,
+				    GBRunObject       *object,
+				    int                property,
+				    GBValue           *val)
 {
-	GBRunObjectPriv *priv;
+	g_warning ("Set arg unimplemented");
+	return FALSE;
+}
+	
+static GBValue *
+gbrun_object_get_arg_unimplemented (GBRunEvalContext  *ec,
+				    GBRunObject       *object,
+				    int                property)
+{
+	g_warning ("Get arg unimplemented");
 
-	g_return_if_fail (obj != NULL);
-	priv = PRIV (obj);
-	g_return_if_fail (priv != NULL);
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (value != NULL);
-
-	make_object_vars (priv);
-
-	gbrun_stack_level_add (ec, priv->vars, name, value);
+	return NULL;
 }
 
 typedef struct {
@@ -228,64 +125,73 @@ setup_vars (gpointer key, gpointer value, gpointer user_data)
 {
 	setup_vars_closure_t *c = user_data;
 	GBVar                *var = value;
-	GBValueType           t;
 	GBValue              *val;
 
 	if (var->is_array)
 		val = gb_value_new_object (GB_OBJECT (gbrun_array_new (c->ec, var)));
+
 	else {
-		if (var->object) {
-			GBObject *obj = gbrun_object_new (c->ec, var->type);
-			val = gb_value_new_object (obj);
-		} else {
-			t   = gb_value_type_from_name (var->type);
-			if (t == GB_VALUE_EMPTY)
-				gb_eval_exception_firev (GB_EVAL_CONTEXT (c->ec),
-							 "Non base type '%s'",
-							 var->type);
-			else
-				val = gb_value_new_default (GB_EVAL_CONTEXT (c->ec), t);
-		}
+		GtkType t;
+		t = gb_gtk_type_from_name (var->type);
+		if (!t)
+			gb_eval_exception_firev (GB_EVAL_CONTEXT (c->ec),
+						 "Unknown type '%s'", var->type);
+		else
+			val = gb_value_new_default (GB_EVAL_CONTEXT (c->ec), t);
 	}
 
-/*	g_warning ("Adding variable '%s' to object %s", var->name,
+/*	g_warning ("Adding variable '%s' to class %s", var->name,
 	gbrun_object_name (c->obj));*/
 
 	gbrun_object_var_add (c->ec, c->obj, var->name, val);
 }
 
-static void
-gbrun_object_add_variables (GBRunEvalContext *ec,
-			    GBRunObject      *obj,
-			    GHashTable       *vars)
+/**
+ * gbrun_object_add_routine:
+ * @klass:   the class to add it to.
+ * @routine: the routine structure
+ * 
+ * Add a VB routine to an existing class.
+ **/
+void
+gbrun_object_add_routine (GBRunObjectClass *klass,
+			  GBRoutine        *routine)
 {
-	setup_vars_closure_t c;
+	GBRunObjMethod *m;
 
-	g_return_if_fail (obj != NULL);
+/*	g_warning ("Adding '%s' to object '%s'\n",
+		   routine->name,
+		   gtk_type_name (GTK_OBJECT_CLASS (klass)->type));*/
 
-	if (!vars)
-		return;
+	g_return_if_fail (klass != NULL);
+	g_return_if_fail (routine != NULL);
 
-	c.ec  = ec;
-	c.obj = obj;
+	m              = g_new0   (GBRunObjMethod, 1);
 
-	g_hash_table_foreach (vars, setup_vars, &c);
+	m->type        = GBRUN_METHOD_VB;
+	m->name        = g_strdup (routine->name);
+	m->args        = g_slist_copy (routine->args);
+	m->handler.vb  = routine;
+	/* FIXME: need to split parse_args code */
+	m->max_args    = g_slist_length (routine->args);
+	m->min_args    = m->max_args;
+	/* No pure VB can have security implications */
+	m->mask        = 0;
+	m->is_sub      = !routine->is_function;
+	
+	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
 }
 
 static void
 add_routine (gpointer key, gpointer value, gpointer user_data)
 {
-	GBRunObjectPrivClass *cl = user_data;
-
-/*	g_warning ("Adding '%s' to object\n", ((GBRoutine *)value)->name);*/
-
-	gbrun_object_add_routine (cl, value);
+	gbrun_object_add_routine (user_data, value);
 }
 
 void
-gbrun_object_add_routines (GBRunEvalContext     *ec,
-			   GBRunObjectPrivClass *klass,
-			   GHashTable           *routines)
+gbrun_object_add_routines (GBRunEvalContext *ec,
+			   GBRunObjectClass *klass,
+			   GHashTable       *routines)
 {
 	g_return_if_fail (ec != NULL);
 	g_return_if_fail (klass != NULL);
@@ -296,66 +202,39 @@ gbrun_object_add_routines (GBRunEvalContext     *ec,
 	g_hash_table_foreach (routines, add_routine, klass);
 }
 
-GBObjectClass *
-gbrun_object_create (const char *vb_name, GSList *parents)
+static void
+gbrun_object_add_variable (GBRunObjectClass *klass,
+			   GBVar            *var)
 {
-	GBObjectClass *oc;
+	/* FIXME: we should dup var really */
+	klass->priv->variables = g_slist_prepend (
+		klass->priv->variables, var);
+}
 
-	if (!parents)
-		parents = g_slist_append (NULL, gbrun_object_class ());
+static void
+add_variables (gpointer key, gpointer value, gpointer user_data)
+{
+	gbrun_object_add_variable (user_data, value);
+}
 
-	if ((oc = gbrun_object_lookup (vb_name))) {
-		g_warning ("re-creating '%s'", vb_name);
-		return oc;
+void
+gbrun_object_add_variables (GBRunEvalContext *ec,
+			    GBRunObjectClass *klass,
+			    GHashTable       *variables)
+{
+	g_return_if_fail (ec != NULL);
+	g_return_if_fail (klass != NULL);
 
-	} else { /* FIXME: leak */
-		GBRunObjectPrivClass *p = g_new0 (GBRunObjectPrivClass, 1);
-
-		gb_object_priv_class_init (&p->priv, vb_name,
-					   sizeof (GBRunObjectPriv),
-					   NULL, NULL);
-		gbrun_object_priv_class_init (p);
-		oc = gb_object_class_new (&p->priv, parents);
-		gbrun_object_register (vb_name, oc);
+	if (!variables) {
+/*		g_warning ("No variables on object '%s'",
+		gtk_type_name (GTK_OBJECT_CLASS (klass)->type));*/
+		return;
 	}
 
-	if (!g_slist_find (oc->parents, gbrun_object_class ()->primary))
-		g_error ("A GBRun object not inheriting correctly");
-		
-	return oc;
-}
+/*	g_warning ("Adding %d variables", g_hash_table_size (variables));*/
 
-GBObjectClass *
-gbrun_object_create_single (const char    *vb_name,
-			    GBObjectClass *parent)
-{
-	GSList        *l;
-	GBObjectClass *ret;
-
-	if (parent)
-		l = g_slist_prepend (NULL, parent);
-	else
-		l = NULL;
-
-	ret = gbrun_object_create (vb_name, l);
-
-	g_slist_free (l);
-
-	return ret;
-}
-
-GBRunObjectPriv *
-gbrun_object_get_priv (GBObject *object)
-{
-	GBObjectClass *klass = GBRUN_OBJECT_GET_CLASS (object);
-
-	return gb_object_get_priv (object, klass);
-}
-
-GBRunObjectPrivClass *
-gbrun_object_class_get_priv (GBObjectClass *klass)
-{
-	return (GBRunObjectPrivClass *)klass->primary;
+	g_hash_table_foreach (variables, 
+			      add_variables, klass);
 }
 
 /*
@@ -363,7 +242,7 @@ gbrun_object_class_get_priv (GBObjectClass *klass)
  */
 GBValue *
 parse_def (const char       *str,
-	   GBValueType       to)
+	   GtkType           to)
 {
 	GBValue          *def_val, *tmp;
 	GBRunEvalContext *ec;
@@ -371,7 +250,7 @@ parse_def (const char       *str,
 	tmp = gb_value_new_string_chars (str);
 	ec  = gbrun_eval_context_new ("Default arg promotion", GBRUN_SEC_HARD);
 
-	def_val = gb_value_promote (GB_EVAL_CONTEXT (ec), tmp, to);
+	def_val = gb_value_promote (GB_EVAL_CONTEXT (ec), to, tmp);
 	gb_value_destroy (tmp);
 
 	if (gbrun_eval_context_exception (ec))
@@ -412,12 +291,9 @@ parse_arg (const char *arg_txt)
 			by_val = FALSE;
 		
 		if (txt [3]) { /* Extract default */
-			GBValueType t = gb_value_type_from_name (txt [1]);
+			GtkType t = gb_gtk_type_from_name (txt [1]);
 			GBValue *def_value;
 			
-			if (t == GB_VALUE_EMPTY)
-				g_error ("Invalid type for a default '%s'", txt [1]);
-
 			if (g_strcasecmp (txt [3], "NULL")) {
 				def_value = parse_def (txt [3], t);
 				if (!def_value)
@@ -542,9 +418,9 @@ parse_arg_desc (GBRunObjMethod *m, const char *arg_desc,
  * an existing class.
  **/
 void
-gbrun_object_add_method_var (GBRunObjectPrivClass  *klass,
-			     const char   *desc,
-			     GBRunMethodVar *method)
+gbrun_object_add_method_var (GBRunObjectClass *klass,
+			     const char       *desc,
+			     GBRunMethodVar   *method)
 {
 	GBRunObjMethod *m;
 
@@ -556,7 +432,7 @@ gbrun_object_add_method_var (GBRunObjectPrivClass  *klass,
 	m->handler.var = method;
 	parse_arg_desc (m, desc, TRUE);
 	
-	klass->methods = g_slist_prepend (klass->methods, m);
+	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
 }
 
 /**
@@ -568,9 +444,9 @@ gbrun_object_add_method_var (GBRunObjectPrivClass  *klass,
  * Add a method to an existing class.
  **/
 void
-gbrun_object_add_method_arg (GBRunObjectPrivClass  *klass,
-			     const char   *desc,
-			     GBRunMethodArg *method)
+gbrun_object_add_method_arg (GBRunObjectClass *klass,
+			     const char       *desc,
+			     GBRunMethodArg   *method)
 {
 	GBRunObjMethod *m;
 
@@ -582,294 +458,113 @@ gbrun_object_add_method_arg (GBRunObjectPrivClass  *klass,
 	m->handler.arg = method;
 	parse_arg_desc (m, desc, FALSE);
 	
-	klass->methods = g_slist_prepend (klass->methods, m);
+	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
 }
 
-/**
- * gbrun_object_add_routine:
- * @klass:   the class to add it to.
- * @routine: the routine structure
- * 
- * Add a VB routine to an existing class.
- **/
-void
-gbrun_object_add_routine (GBRunObjectPrivClass  *klass,
-			  GBRoutine    *routine)
-{
-	GBRunObjMethod *m;
-
-	g_return_if_fail (klass != NULL);
-	g_return_if_fail (routine != NULL);
-
-	m              = g_new0   (GBRunObjMethod, 1);
-
-	m->type        = GBRUN_METHOD_VB;
-	m->name        = g_strdup (routine->name);
-	m->args        = g_slist_copy (routine->args);
-	m->handler.vb  = routine;
-	/* FIXME: need to split parse_args code */
-	m->max_args    = g_slist_length (routine->args);
-	m->min_args    = m->max_args;
-	/* No pure VB can have security implications */
-	m->mask        = 0;
-	m->is_sub      = !routine->is_function;
-	
-	klass->methods = g_slist_prepend (klass->methods, m);
-}
-
-GBRunObjProperty *
-gbrun_object_add_property_val (GBRunObjectPrivClass *klass,
-			       const char  *name,
-			       GBValueType  type)
-{
-	GBRunObjProperty *p;
-
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (klass != NULL, NULL);
-
-	p              = g_new0   (GBRunObjProperty, 1);
-	p->idx         = 0; /* FIXME */
-	p->get_arg     = klass->get_arg;
-	p->set_arg     = klass->set_arg;
-
-	/* It helps if it is settable */
-	g_assert (p->get_arg || p->set_arg);
-
-	p->type        = GBRUN_PROP_VAL;
-	p->name        = g_strdup (name);
-	p->u.prop_type = type;
-	
-	klass->properties = g_list_prepend (klass->properties, p);
-
-	return p;
-}
-
-GBRunObjProperty *
-gbrun_object_add_property_obj (GBRunObjectPrivClass *klass,
-			       const char  *name,
-			       const char  *objname)
-{
-	GBRunObjProperty *p;
-
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (klass != NULL, NULL);
-	g_return_val_if_fail (objname != NULL, NULL);
-
-	p            = g_new0   (GBRunObjProperty, 1);
-	p->idx       = 0; /* FIXME */
-	p->get_arg   = klass->get_arg;
-	p->set_arg   = klass->set_arg;
-
-	/* It helps if it is settable */
-	g_assert (p->get_arg || p->set_arg);
-
-	p->type      = GBRUN_PROP_OBJ;
-	p->name      = g_strdup (name);
-	p->u.objname = g_strdup (objname);
-	
-	klass->properties = g_list_prepend (klass->properties, p);
-
-	return p;
-}
-
-GBRunObjMethod *
+static GBRunObjMethod *
 gbrun_object_get_method (GBRunObjectClass *klass,
 			 const char       *name)
 {
 	GSList *l, *i;
-	GSList *privs = g_slist_reverse (g_slist_copy (klass->parents));
 
-	for (i = privs; i; i = i->next) {
-		GBRunObjectPrivClass *pc = i->data;
+	g_return_val_if_fail (klass != NULL, NULL);
 
-		for (l = pc->methods; l; l = l->next) {
+	for (i = klass->priv->classes; i; i = i->next) {
+		GBRunObjectClass *cur_class = i->data;
+
+/*		g_warning ("Searching class of type '%s' for '%s'",
+			   gtk_type_name (GTK_OBJECT_CLASS (cur_class)->type),
+			   name);*/
+
+		for (l = cur_class->priv->methods; l; l = l->next) {
 			GBRunObjMethod *m = l->data;
-			
+
 			if (!g_strcasecmp (m->name, name))
 				return m;
 		}
 	}
-	g_slist_free (privs);
 
 	return NULL;
 }
 
 GSList *
-gbrun_object_get_methods (GBObjectClass *klass)
+gbrun_object_get_methods (GBRunObjectClass *klass)
+{
+	g_return_val_if_fail (klass != NULL, NULL);
+	
+	return g_slist_copy (klass->priv->methods);
+}
+
+gboolean
+gbrun_object_has_method (GBRunObjectClass *klass,
+			 const char       *name)
+{
+	if (gbrun_object_get_method (klass, name))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+typedef struct {
+	GtkType type;
+	int     idx;
+	char   *name;
+} GBRunObjectProperty;
+
+static GBRunObjectProperty *
+gbrun_object_get_property (GBRunObjectClass  *klass,
+			   const char        *name,
+			   GBRunObjectClass **on_class)
 {
-	GSList *ans = NULL;
 	GSList *l, *i;
-	GSList *privs;
 
 	g_return_val_if_fail (klass != NULL, NULL);
 
-	privs = g_slist_reverse (g_slist_copy (klass->parents));
+	for (i = klass->priv->classes; i; i = i->next) {
+		GBRunObjectClass *cur_class = i->data;
+	
+		*on_class = cur_class;
 
-	for (i = privs; i; i = i->next) {
-		GBRunObjectPrivClass *pc = i->data;
-
-		for (l = pc->methods; l; l = l->next) {
-			ans = g_slist_prepend (ans, l->data);
+		for (l = cur_class->priv->properties; l; l = l->next) {
+			GBRunObjectProperty *prop = l->data;
+			
+			if (!g_strcasecmp (prop->name, name))
+				return prop;
 		}
 	}
-
-	return g_slist_reverse (ans);
-}
-
-static GBRunObjProperty *
-get_class_prop (GBRunObjectPrivClass *pc,
-		const char           *name)
-{
-	GList *l;
-	
-	for (l = pc->properties; l; l = l->next) {
-		GBRunObjProperty *p = l->data;
+	*on_class = NULL;
 		
-		if (!g_strcasecmp (p->name, name))
-			return p;
-	}
-
-	return NULL;	
-}
-
-GBRunObjProperty *
-gbrun_object_get_property (GBRunObjectClass      *klass,
-			   const char            *name)
-{
-	GSList *l, *privs = g_slist_reverse (g_slist_copy (klass->parents));
-
-	for (l = privs; l; l = l->next) {
-		GBRunObjectPrivClass *pc = l->data;
-		GBRunObjProperty     *p;
-		
-		if ((p = get_class_prop (pc, name)))
-			return p;
-	} 
-
 	return NULL;
 }
 
 void
-gbrun_object_globalize (GBObject *klass)
+gbrun_object_add_property (GBRunObjectClass *klass,
+			   const char       *name,
+			   GtkType           type,
+			   int               idx)
 {
-	gbrun_global_objects = g_list_prepend (gbrun_global_objects, klass);
+	GBRunObjectProperty *prop;
+
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (klass != NULL);
+
+	prop = g_new0 (GBRunObjectProperty, 1);
+
+	prop->type = type;
+	prop->name = g_strdup (name);
+	prop->idx  = idx;
+
+	klass->priv->properties =
+		g_slist_prepend (klass->priv->properties, prop);
 }
 
-void
-gbrun_object_priv_class_init (GBRunObjectPrivClass *pc)
+gboolean
+gbrun_object_has_property (GBRunObjectClass *klass,
+			   const char       *name)
 {
-	pc->vars       = NULL;
+	GBRunObjectClass *dummy;
 
-	pc->name       = NULL;
-	pc->properties = NULL;
-	pc->methods    = NULL;
-
-	pc->set_arg    = NULL;
-	pc->get_arg    = NULL;
-}
-
-/**
- * gbrun_object_global_func:
- * @name: 
- * 
- *  This returns both functions & naff cretinous unscoped
- * object methods such as 'Debug.print'.
- * 
- * Return value: 
- **/
-GBRunObjMethod *
-gbrun_object_global_func (const char *name)
-{
-	GBRunObjMethod *m = NULL;
-	GList        *l;
-
-	for (l = gbrun_global_objects; l; l = l->next) {
-		GBRunObjectClass *klass;
-
-		klass = GBRUN_OBJECT_GET_CLASS (l->data);
-		if ((m = gbrun_object_get_method (klass, name)))
-			break;
-	}
-
-	return m;
-}
-
-const char *
-gbrun_object_name (GBRunObject *object)
-{
-	GBRunObjectClass     *klass;
-	GBRunObjectPrivClass *pc;
-
-	g_return_val_if_fail (GB_IS_OBJECT (object), "Non GB object");
-
-	klass = GBRUN_OBJECT_GET_CLASS (object);
-
-	pc = gbrun_object_class_get_priv (klass);
-
-	return pc->name;
-}
-
-/*
- * End of Class maniupulators, start of object manipulators.
- */ 
-
-void
-gbrun_object_set_arg (GBRunEvalContext *ec,
-		      GBRunObject      *object,
-		      const char       *prop_name,
-		      GBValue          *val)
-{
-	GBRunObjProperty     *prop;
-	GBValue              *v;
-
-	g_return_if_fail (ec != NULL);
-	g_return_if_fail (val != NULL);
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (prop_name != NULL);
-	g_return_if_fail (GB_IS_OBJECT (object));
-
-	prop = gbrun_object_get_property (GBRUN_OBJECT_GET_CLASS (object), prop_name);
-
-	if (!prop || !prop->set_arg) {
-		gbrun_exception_firev (ec, "No writeable property '%s' on object '%s'",
-				       prop_name, gbrun_object_name (object));
-		return;
-	}
-
-	if (prop->type == GBRUN_PROP_VAL) {
-		v = gbrun_value_promote (ec, val, prop->u.prop_type);
-
-		if (!v)
-			return;
-
-		prop->set_arg (ec, object, prop, v);
-
-		gb_value_destroy (v);
-	} else {
-		g_warning ("FIXME: untested object setting ...");
-		prop->set_arg (ec, object, prop, val);
-	}
-}
-
-GBValue *
-gbrun_object_get_arg (GBRunEvalContext *ec,
-		      GBRunObject      *object,
-		      const char       *prop_name)
-{
-	GBRunObjProperty     *prop;
-
-	g_return_val_if_fail (ec != NULL, NULL);
-	g_return_val_if_fail (object != NULL, NULL);
-	g_return_val_if_fail (prop_name != NULL, NULL);
-	g_return_val_if_fail (GB_IS_OBJECT (object), NULL);
-
-	prop = gbrun_object_get_property (GBRUN_OBJECT_GET_CLASS (object), prop_name);
-
-	if (!prop || !prop->get_arg)
-		return gbrun_exception_firev (ec, "No reaable property '%s' on object '%s'",
-					      prop_name, gbrun_object_name (object));
-	
-	return prop->get_arg (ec, object, prop);
+	return gbrun_object_get_property (klass, name, &dummy) != NULL;
 }
 
 static GBValue *
@@ -961,33 +656,44 @@ gbrun_method_invoke_var (GBRunEvalContext *ec, GBRunObject      *obj,
 	return m->handler.var (ec, obj, func->parms); 
 }
 
-void
-stack_setup_vars (gpointer key, gpointer value, gpointer user_data)
+static GBValue *
+value_from_var (GBRunEvalContext *ec,
+		GBVar            *var)
 {
-	GBRunEvalContext *ec  = user_data;
-	GBVar          *var = value;
-	GBValueType     t;
-	GBValue        *val;
+	GBValue *val;
 
 	if (var->is_array)
 		val = gb_value_new_object (GB_OBJECT (gbrun_array_new (ec, var)));
 	else {
+		GtkType t;
+
 		if (var->object) {
-			GBObject *obj = gbrun_object_new (ec, var->type);
-			val = gb_value_new_object (obj);
+			t = gb_gtk_type_from_name (var->type);
+			val = gb_value_new_object (gtk_type_new (t));
+
 		} else {
-			t   = gb_value_type_from_name (var->type);
-			if (t == GB_VALUE_EMPTY)
+			t = gb_gtk_type_from_name (var->type);
+
+			if (!t)
 				gb_eval_exception_firev (GB_EVAL_CONTEXT (ec),
-							 "Non base type '%s'", var->type);
+							 "Unknown type '%s'", var->type);
 			else
 				val = gb_value_new_default (GB_EVAL_CONTEXT (ec), t);
 		}
 	}
 
+	return val;
+}
+
+static void
+stack_setup_vars (gpointer key, gpointer value, gpointer user_data)
+{
+	GBVar   *var = value;
+	GBValue *val = value_from_var (user_data, var);
+
 /*	g_warning ("Adding variable '%s' at stack level %d", var->name,
 	g_list_length (ec->stack->level));*/
-	gbrun_stack_set (ec, var->name, val);
+	gbrun_stack_set (user_data, var->name, val);
 }
 
 static GBValue *
@@ -998,7 +704,7 @@ gbrun_method_invoke_vb (GBRunEvalContext *ec, GBRunObject      *obj,
 	GSList      *l;
 	GSList      *def;
 	GSList      *parms;
-	GBRunObject *old_me;
+	GBObject    *old_me;
 	int          len;
 
 	g_return_val_if_fail (m != NULL, NULL);
@@ -1016,7 +722,7 @@ gbrun_method_invoke_vb (GBRunEvalContext *ec, GBRunObject      *obj,
 
 	/* Set 'Me' up */
 	old_me = gbrun_eval_context_me_get (ec);
-	gbrun_eval_context_me_set (ec, obj);
+	gbrun_eval_context_me_set (ec, GB_OBJECT (obj));
 
 #ifdef VB_METHOD_DEBUG
 	fprintf (stderr, "Calling %s (", func->name?func->name:"[Unknown]");
@@ -1103,114 +809,393 @@ gb_cleanup:
 	return ans;
 }
 
-static GBRunObjMethod *
-get_method (GBRunEvalContext *ec, GBRunObject **obj, const char *name)
+static void
+gbrun_object_copy (GBEvalContext  *ec,
+		   GBObject       *src,
+		   GBObject       *dest)
 {
-	GBRunObjMethod *m = NULL;
-	GBRunProject   *proj;
+	g_warning ("We badly need to copy all the object variables here");
+}
+
+static gboolean
+chain_assign_to_value (GBEvalContext  *ec,
+		       GBValue        *object,
+		       const GBObjRef *ref,
+		       GBValue        *value,
+		       gboolean        try_assign)
+{
+	g_return_val_if_fail (object != NULL, FALSE);
+
+	if (GB_IS_AN_OBJECT (object->gtk_type)) {
+		GBObjRef newref;
+		gboolean ret;
+		
+		newref = *ref;
+		newref.name = NULL;
+
+		ret = gb_object_assign (
+			ec, object->v.obj, &newref, value, try_assign);
+		gb_value_destroy (object);
+		
+		return ret;
+	} else {
+		gb_value_destroy (object);
+		
+		if (!try_assign)
+			gbrun_exception_firev (
+				GBRUN_EVAL_CONTEXT (ec),
+				"Cannot use parameters on '%s'", ref->name);
+	}
+	return FALSE;
+}
+
+
+static gboolean
+gbrun_object_assign (GBEvalContext  *ec,
+		     GBObject       *object,
+		     const GBObjRef *ref,
+		     GBValue        *value,
+		     gboolean        try_assign)
+{
+	GBRunObjectProperty *prop;
+	GBRunObjectClass *klass;
+	gboolean ret;
+	GBValue *v, *sval;
+
+	if (!object)
+		return FALSE;
+
+	if ((sval = gbrun_object_var_get (GBRUN_EVAL_CONTEXT (ec),
+					  GBRUN_OBJECT (object),
+					  ref->name))) {
+
+		if (ref->parms)
+			return chain_assign_to_value (
+				ec, sval, ref, value, try_assign);
+
+		gbrun_object_var_set (GBRUN_EVAL_CONTEXT (ec),
+				      GBRUN_OBJECT (object),
+				      ref->name, value);
+		gb_value_destroy (sval);
+
+		return TRUE;
+	}
+
+	prop = gbrun_object_get_property (
+		GBRUN_OBJECT_GET_CLASS (object), ref->name, &klass);
+
+	if (!prop) {
+		if (!try_assign)
+			gbrun_exception_firev (
+				GBRUN_EVAL_CONTEXT (ec),
+				"No writeable property '%s' on object '%s'",
+				ref->name, gbrun_object_name (GBRUN_OBJECT (object)));
+		return FALSE;
+	}
+
+	v = gb_value_promote (GB_EVAL_CONTEXT (ec), prop->type, value);
+
+	if (!v)
+		return FALSE;
+	
+	ret = klass->set_arg (GBRUN_EVAL_CONTEXT (ec),
+			      GBRUN_OBJECT (object), prop->idx, v);
+	
+	gb_value_destroy (v);
+
+	return ret;
+}
+
+static GBValue *
+gbrun_object_deref (GBEvalContext  *gb_ec,
+		    GBObject       *obj,
+		    const GBObjRef *ref,
+		    gboolean        try_deref)
+{
+	GBRunEvalContext *ec;
+	GBValue          *val = NULL;
+	GBRunObjMethod   *m;
 
-	g_return_val_if_fail (ec != NULL, NULL);
 	g_return_val_if_fail (obj != NULL, NULL);
 
-	if (!*obj)
-		*obj = gbrun_eval_context_me_get (ec);
+	ec = GBRUN_EVAL_CONTEXT (gb_ec);
 
-	if (*obj)
-		m = gbrun_object_get_method (GBRUN_OBJECT_GET_CLASS (*obj), name);
+/*	g_warning ("Object deref '%s' method '%d' params: %d",
+	ref->name, ref->method, g_slist_length (ref->parms));*/
 
-	if (m)
-		return m;
+	/* 1. See if we have an object variable */
+	if ((val = gbrun_object_var_get (GBRUN_EVAL_CONTEXT (ec),
+					 GBRUN_OBJECT (obj),
+					 ref->name))) {
+		if (ref->parms) { /* Chain to array dereference */
+			if (GB_IS_AN_OBJECT (val->gtk_type)) {
+				GBObjRef newref;
+				GBValue *ret;
 
-	if ((proj = gbrun_eval_context_proj_get (ec))) {
+				newref = *ref;
+				newref.name = NULL;
+				ret = gb_object_deref (
+					gb_ec, val->v.obj, &newref, try_deref);
+				gb_value_destroy (val);
 
-		GSList *l = gbrun_project_get_modules (proj);
+				return ret;
+			} else {
+				gb_value_destroy (val);
 
-		while (l) {
-			*obj = l->data;
+				if (try_deref)
+					goto check_method;
+				else
+					gbrun_exception_firev (
+						ec,
+						"Cannot use parameters on '%s'", ref->name);
+				return NULL;
+			}
+		} else
+			return val;
+	}
 
-			m = gbrun_object_get_method (GBRUN_OBJECT_GET_CLASS (*obj), name);
-			if (m)
-				return m;
+ check_method:
+	/* 2. See if we have a method */
+	if ( !(m = gbrun_object_get_method (
+		GBRUN_OBJECT_GET_CLASS (obj), ref->name)) ) {
 
-			l = l->next;
+		/* 3. Fall back to an argument */
+		if (!ref->method && !ref->parms) {
+			GBRunObjectProperty *prop;
+			GBRunObjectClass    *klass;
+
+			if (!obj)
+				return NULL;
+
+			prop = gbrun_object_get_property (
+				GBRUN_OBJECT_GET_CLASS (obj),
+				ref->name, &klass);
+
+			if (!prop) {
+				if (!try_deref)
+					gbrun_exception_firev (
+						ec, "No readable property '%s' on object '%s'",
+						ref->name, gbrun_object_name (GBRUN_OBJECT (obj)));
+				return NULL;
+			}
+
+			return klass->get_arg (ec, GBRUN_OBJECT (obj), prop->idx);
+
+		} else {
+			if (!try_deref)
+				gbrun_exception_firev (
+					ec, "No '%s' method / function", ref->name);
+			return NULL;
 		}
 	}
 
-	*obj = NULL;
-
-	return gbrun_object_global_func (name);
-}
-
-GBValue *
-gbrun_method_invoke (GBRunEvalContext *ec, GBRunObject *obj,
-		     const GBObjRef *func)
-{
-	GBValue        *val = NULL;
-	GBRunObjMethod *m;
-
-	if (obj && GBRUN_IS_ARRAY (obj))
-		return gbrun_array_deref (ec, obj, func);
-
-	if (!(m = get_method (ec, &obj, func->name)))
-		return gbrun_exception_firev (ec, "No '%s' method / function",
-					    func->name);
-
-	if (ec->flags & m->mask)
-		return gbrun_exception_firev (ec, "Security block on function '%s'",
-					    func->name);
+	if (ec->flags & m->mask) {
+		if (!try_deref)
+			gbrun_exception_firev (
+				ec, "Security block on function '%s'", ref->name);
+		return NULL;
+	}
 
+	/*
+	 * FIXME: what if the method doesn't take
+	 * parameters and returns an array ?
+	 */
 	switch (m->type) {
+
 	case GBRUN_METHOD_ARG:
-		val = gbrun_method_invoke_arg (ec, obj, m, func);
+		val = gbrun_method_invoke_arg (
+			ec, GBRUN_OBJECT (obj), m, ref);
 		break;
 
 	case GBRUN_METHOD_VAR:
-		val = gbrun_method_invoke_var (ec, obj, m, func);
+		val = gbrun_method_invoke_var (
+			ec, GBRUN_OBJECT (obj), m, ref);
 		break;
 
 	case GBRUN_METHOD_VB:
-		val = gbrun_method_invoke_vb  (ec, obj, m, func);
+		val = gbrun_method_invoke_vb  (
+			ec, GBRUN_OBJECT (obj), m, ref);
 		break;
 	}
 
 	return val;
 }
 
-static gint
-g_str_case_equal (gconstpointer v, gconstpointer v2)
+static void
+gbrun_object_destroy (GtkObject *object)
 {
-	  return g_strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
+	GBRunObject *obj = (GBRunObject *) object;
+
+	if (obj->priv) {
+		if (obj->priv->vars) {
+			gbrun_stack_level_destroy (obj->priv->vars);
+			obj->priv->vars = NULL;
+		}
+		g_free (obj->priv);
+	}
+	obj->priv = NULL;
 }
 
-static guint
-g_str_case_hash (gconstpointer v)
+static void
+gbrun_object_instance_init (GBRunObject      *object,
+			    GBRunObjectClass *klass)
 {
-	const char *s = (char*)v;
-	const char *p;
-	guint h=0, g;
-	
-	for (p = s; *p != '\0'; p += 1) {
-		h = ( h << 4 ) + tolower (*p);
-		if ( ( g = h & 0xf0000000 ) ) {
-			h = h ^ (g >> 24);
-			h = h ^ g;
-		}
+	GSList *l;
+
+	/* FIXME: we badly need our own init fn ! */
+	GBRunEvalContext *ec = gbrun_eval_context_new (
+		"brutal-instance-init-hack", GBRUN_SEC_HARD);
+
+	object->priv = g_new0 (GBRunObjectPrivate, 1);
+
+	object->priv->vars = gbrun_stack_level_new ("Object vars");
+
+/*	g_warning ("Instance init '%s' ( '%d' vars )",
+		   gtk_type_name (GTK_OBJECT_CLASS (klass)->type),
+		   g_slist_length (klass->priv->variables));*/
+
+	/* Setup the object's data */
+	for (l = klass->priv->variables; l; l = l->next) {
+
+		GBVar   *var = l->data;
+		GBValue *val = value_from_var (ec, var);
+
+/*		g_warning ("Adding instance object variable '%s'", var->name);*/
+
+		gbrun_stack_level_add (ec, object->priv->vars,
+				       var->name, val);
 	}
-	
-	return h /* % M */;
+
+	gtk_object_unref (GTK_OBJECT (ec));
+}
+
+/*
+ * FIXME:
+ *   Ok; so when we sub-class we need to _copy_ the
+ * data on the ( same ) class of the parent that we
+ * are 'sub-classing'; should be fun / impossible :-)
+ */
+
+static GSList *
+build_classes (GBRunObjectClass *klass)
+{
+	GSList *ret;
+	GtkType type = GTK_OBJECT_CLASS (klass)->type;
+
+	ret = g_slist_append (NULL, klass);
+
+	if (GBRUN_IS_AN_OBJECT (type) &&
+	    type != GBRUN_TYPE_OBJECT) /* we are abstract */
+		ret = g_slist_concat (ret, build_classes (
+			gtk_type_parent_class (type)));
+
+	return ret;
+}
+
+static void
+gbrun_object_class_init (GBRunObjectClass *klass)
+{
+	GBObjectClass  *object_class = (GBObjectClass *) klass;
+	GtkObjectClass *gtk_class    = (GtkObjectClass *) klass;
+
+	klass->priv = g_new0 (GBRunObjectPrivateClass, 1);
+
+	klass->priv->classes = build_classes (klass);
+
+	klass->set_arg = gbrun_object_set_arg_unimplemented;
+	klass->get_arg = gbrun_object_get_arg_unimplemented;
+
+	object_class->copy   = gbrun_object_copy;
+	object_class->assign = gbrun_object_assign;
+	object_class->deref  = gbrun_object_deref;
+
+	gtk_class->destroy   = gbrun_object_destroy;
+}
+
+GtkType
+gbrun_object_get_type (void)
+{
+	static GtkType object_type = 0;
+
+	if (!object_type) {
+		static const GtkTypeInfo object_info = {
+			"GBRunObject",
+			sizeof (GBRunObject),
+			sizeof (GBRunObjectClass),
+			(GtkClassInitFunc)  NULL,
+			(GtkObjectInitFunc) gbrun_object_instance_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) gbrun_object_class_init
+		};
+
+		object_type = gtk_type_unique (GB_TYPE_OBJECT, &object_info);
+		gtk_type_class (object_type);
+	}
+
+	return object_type;	
+}
+
+GtkType
+gbrun_object_subclass (GtkType           parent,
+		       const char       *vb_name,
+		       GtkClassInitFunc  class_init,
+		       GtkObjectInitFunc object_init)
+{
+	char   *gtk_name;
+	GtkType type;
+
+	gtk_name = gb_gtk_type_name (vb_name);
+
+	if (!(type = gtk_type_from_name (gtk_name))) {
+		GtkTypeInfo   object_info;
+		GtkTypeQuery *query;
+
+		query = gtk_type_query (parent);
+		g_return_val_if_fail (query != NULL, 0);
+
+		object_info.type_name   = (char *) gtk_name;
+		object_info.object_size = query->object_size;
+		object_info.class_size  = query->class_size;
+		object_info.class_init_func  = NULL;
+		object_info.object_init_func = object_init;
+		object_info.reserved_1 = NULL;
+		object_info.reserved_2 = NULL;
+		object_info.base_class_init_func = class_init;
+
+		type = gtk_type_unique (parent, &object_info);
+		gtk_type_class (type);
+	}
+
+	g_free (gtk_name);
+
+	return type;
+}
+
+GtkType
+gbrun_object_subclass_simple (GtkType     parent,
+			      const char *vb_name)
+{
+	return gbrun_object_subclass (parent, vb_name, NULL, NULL);
+}
+
+char *
+gbrun_object_name (GBRunObject *object)
+{
+	g_return_val_if_fail (GBRUN_IS_OBJECT (object),
+			      g_strdup ("Null object"));
+
+	return gb_type_name_from_gtk (
+		gtk_type_name (GTK_OBJECT (object)->klass->type));
 }
 
 void
 gbrun_object_init (GBEvalContext *ec)
 {
-	gbrun_object_hash = g_hash_table_new (g_str_case_hash, g_str_case_equal);
+	gbrun_global_get ();
 }
 
 void 
 gbrun_object_shutdown ()
 {
-	/* FIXME: For now, just do this */
-	/* Later, each object's destructor would have to be called, I guess */
-
-	g_hash_table_destroy (gbrun_object_hash);
 }
