@@ -1,4 +1,3 @@
-
 /*
  * GNOME Basic Enterpreter library main.
  *
@@ -19,11 +18,13 @@
 #include <gbrun/objects/gba/libgba.h>
 #include <gbrun/objects/gbrun-objects.h>
 
+#define HAVE_FORM_CODE
+
 struct _GBRunProjectPriv {
 	GBRunStreamProvider *provider;
 	gpointer             user_data;
 	GBProject           *gb_proj;
-	GSList              *modules;
+	GSList              *objects;
 
 	GHashTable          *classes;
 };
@@ -37,11 +38,11 @@ project_destroy (GtkObject *obj)
 	gb_project_destroy (priv->gb_proj);
 	priv->gb_proj = NULL;
 
-	while (priv->modules) {
-		GBObject *obj = priv->modules->data;
+	while (priv->objects) {
+		GBObject *obj = priv->objects->data;
 
 		gb_object_unref (obj);
-		priv->modules = g_slist_remove (priv->modules, obj);
+		priv->objects = g_slist_remove (priv->objects, obj);
 	}
 
 	g_free (priv);
@@ -92,45 +93,44 @@ object_from_data (GBRunEvalContext  *ec,
 		  const char        *name,
 		  const GBParseData *pd)
 {
-	GBRunObject          *obj;
-	GBObjectClass        *klass;
-	GBRunObjectPrivClass *pc;
-	const char           *objname;
+	GBRunObject      *obj;
+	GBRunObjectClass *klass;
+	GtkType           type;
 
 	g_return_val_if_fail (ec != NULL, NULL);
 	g_return_val_if_fail (pd != NULL, NULL);
-
 	
 	if (pd->form) {
-		objname = pd->form->name;
-		klass = gbrun_object_create_single (objname,
-						    gbrun_form_class ());
-	} else {
-		objname = name;
-		klass = gbrun_object_create_single (objname,
-						    gbrun_object_class ());
-	}
+#ifdef HAVE_FORM_CODE
+		type = gbrun_object_subclass_simple (
+			gbrun_form_get_type (), pd->form->name);
+#else
+	g_warning ("No form code");
+	return NULL;
+#endif
+	} else
+		type = gbrun_object_subclass_simple (
+			gbrun_object_get_type (), name);
 
-	{
-		GSList *l;
-		/* FIXME: Ugly hack to setup data */
-		for (l = klass->parents; l; l = l->next) {
-			pc = (GBRunObjectPrivClass *)l->data;
-			if (!g_strcasecmp (pc->priv.name,
- 					   "--GBRun.Object--"))
-				pc->vars = pd->variables;
-		}
-	}
-	obj = gbrun_object_new (ec, objname);
-	
+	klass = gtk_type_class (type);
+	g_return_val_if_fail (klass != NULL, NULL);
+
+	gbrun_object_add_routines (ec, klass, pd->routines);
+	gbrun_object_add_variables (ec, klass, pd->variables);
+
+	obj = gtk_type_new (type);
+
+	gbrun_eval_context_me_set (ec, GB_OBJECT (obj));
+
+#ifdef HAVE_FORM_CODE	
 	if (pd->form)
-		gbrun_form_init (ec, obj, pd);
-
-	pc = (GBRunObjectPrivClass *)GB_OBJECT_PRIV_GET_CLASS (gbrun_object_get_priv (obj));
-	gbrun_object_add_routines  (ec, pc, pd->routines);
+		gbrun_form_init (ec, GBRUN_FORM (obj), pd);
+#endif
 
+#ifdef HAVE_FORM_CODE	
 	if (pd->form)
-		gbrun_form_invoke (ec, obj, "Form_Load");
+		gbrun_form_invoke (ec, GBRUN_FORM (obj), "Form_Load");
+#endif
 
 	return obj;
 }
@@ -180,7 +180,7 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 	for (l = p->modules; l; l = l->next) {
 		GBProjectPair     *pp = l->data;
 		const GBParseData *pd;
-		GBObject          *obj;
+		GBRunObject       *obj;
 
 		fprintf (stderr, "Loading module '%s'\n", pp->filename);
 
@@ -190,13 +190,13 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		obj = object_from_data (ec, pp->filename, pd);
 		gb_parse_data_destroy (pd);
 
-		priv->modules = g_slist_append (priv->modules, obj);
+		priv->objects = g_slist_append (priv->objects, obj);
 	}
 
 	for (l = p->classes; l; l = l->next) {
 		GBProjectPair     *pp = l->data;
 		const GBParseData *pd;
-		GBObject          *obj;
+		GBRunObject       *obj;
 
 		fprintf (stderr, "Loading class '%s'\n", pp->filename);
 
@@ -213,7 +213,7 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 	for (l = p->forms; l; l = l->next) {
 		char              *filename = l->data;
 		const GBParseData *pd;
-		GBObject          *obj;
+		GBRunObject       *obj;
 
 		fprintf (stderr, "Loading form '%s'\n", (char *)l->data);
 
@@ -284,13 +284,31 @@ gbrun_shutdown (void)
 	gbrun_object_shutdown  ();
 }
 
-GSList *
-gbrun_project_get_modules (GBRunProject *proj)
+const GSList *
+gbrun_project_get_objects (GBRunProject *proj)
 {
-	g_return_val_if_fail (proj != NULL, NULL);
-	g_return_val_if_fail (proj->priv != NULL, NULL);
+	g_return_val_if_fail (GBRUN_IS_PROJECT (proj), NULL);
 
-	return proj->priv->modules;
+	return proj->priv->objects;
+}
+
+void
+gbrun_project_register_object (GBRunProject *proj,
+			       GBRunObject  *object)
+{
+	g_return_if_fail (GBRUN_IS_PROJECT (proj));
+
+	if (!g_slist_find (proj->priv->objects, object))
+		proj->priv->objects = g_slist_append (proj->priv->objects, object);
+}
+
+void
+gbrun_project_deregister_object (GBRunProject *proj,
+				 GBRunObject  *object)
+{
+	g_return_if_fail (GBRUN_IS_PROJECT (proj));
+
+	proj->priv->objects = g_slist_remove (proj->priv->objects, object);
 }
 
 GBValue *
@@ -318,8 +336,9 @@ gbrun_project_invoke (GBRunEvalContext *ec, GBRunProject *proj,
 
 	exprs = g_slist_reverse (exprs);
 	ref.parms = exprs;
+	ref.method = TRUE;
 
-	val = gbrun_method_invoke (ec, NULL, &ref);
+	val = gbrun_objref_deref (ec, NULL, &ref, TRUE);
 
 	while (exprs) {
 		gb_expr_destroy (exprs->data);
@@ -340,7 +359,7 @@ gbrun_project_fn_names (GBRunProject *proj)
 	g_return_val_if_fail (proj != NULL, NULL);
 	g_return_val_if_fail (proj->priv != NULL, NULL);
 
-	for (l = proj->priv->modules; l; l = l->next) {
+	for (l = proj->priv->objects; l; l = l->next) {
 		GBRunObject          *obj = l->data;
 		GSList               *m, *i;
 
