@@ -1,4 +1,3 @@
-
 /*
  * GNOME Basic project runtime
  *
@@ -32,7 +31,11 @@ struct _GBRunProjectClass {
 	GtkObjectClass     klass;
 };
 
-GtkType           gbrun_project_get_type    (void);
-GSList           *gbrun_project_get_modules (GBRunProject *);
+GtkType           gbrun_project_get_type          (void);
+const GSList     *gbrun_project_get_objects       (GBRunProject *project);
+void              gbrun_project_register_object   (GBRunProject *project,
+						   GBRunObject  *object);
+void              gbrun_project_deregister_object (GBRunProject *project,
+						   GBRunObject  *object);
 
 #endif /* GBRUN_PROJECT_H */
