@@ -447,7 +447,7 @@ gbrun_stmt_for (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
 		if (!step)
 			return FALSE;
 
-		tmp = gb_eval_add (gbrun_objref_deref (ec, NULL, &i), step);
+		tmp = gb_eval_add (gbrun_objref_deref (ec, NULL, &i, TRUE), step);
 		gbrun_stack_set (ec, i.name, tmp);
 
 		gb_value_destroy (tmp);
@@ -458,7 +458,7 @@ gbrun_stmt_for (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
 				   stmt->parm.forloop.to);
 	if (!to ||
 	    !gb_eval_compare (GB_EVAL_CONTEXT (ec),
-			      gbrun_objref_deref (ec, NULL, &i),
+			      gbrun_objref_deref (ec, NULL, &i, TRUE),
 			      GB_EXPR_LE, to, &ok)) {
 		gb_value_destroy (to);
 		return FALSE;
