@@ -13,40 +13,24 @@
 
 #include "gbrun-form-item.h"
 
-#define ITEM_NAME "VB.CommandButton"
+#define ITEM_NAME "gb-vb.commandbutton"
 
-struct _GBRunCmdButton {
-	GBRunObjectPriv priv;
+enum {
+	ARG_FIRST = 0,
+	CAPTION,
+	TOOLTIP_TEXT,
+	ENABLED,
+	DEFAULT
+};
+
+#define GBRUN_CMDBUTTON(obj) (GTK_CHECK_CAST ((obj), gbrun_cmdbutton_get_type (), GBRunCmdButton))
+
+typedef struct {
+	GBRunFormItem   item;
 
 	GtkLabel       *label;
 	char           *real_caption;
-};
-
-static void copy     (GBEvalContext *, GBObject *, GBObject *);
-static void destruct (GBObject *object);
-
-static GBObjectClass *
-cmdbutton_class (void)
-{
-	static GBObjectClass *oc = NULL;
-
-	if (!oc) {
-		static GBRunObjectPrivClass p;
-		gb_object_priv_class_init (&p.priv, ITEM_NAME,
-					   sizeof (GBRunCmdButton),
-					   (GBObjectCopy *)copy,
-					   (GBObjectDestructor *)destruct);
-		gbrun_object_priv_class_init (&p);
-
-		oc = gb_object_class_new_single (&p.priv, gbrun_form_item_class ());
-		gbrun_object_register (ITEM_NAME, oc);
-	}
-
-	return oc;
-}
-
-#define PRIV(o) (gb_object_get_priv ((o), cmdbutton_class ()))
-
+} GBRunCmdButton;
 
 /*
  * _GBECmdButton defines a VB Command Button
@@ -87,11 +71,6 @@ cmdbutton_class (void)
  *
  */
 
-static GBRunObjProperty *p_caption       = NULL;
-static GBRunObjProperty *p_tool_tip_text = NULL;
-static GBRunObjProperty *p_enabled       = NULL;
-static GBRunObjProperty *p_default       = NULL;
-
 /**
  * cmdbutton_click
  *   @button
@@ -118,19 +97,120 @@ cmdbutton_mouseup (GtkWidget *button, GBRunEvalContext *ec)
 	gbrun_form_item_invoke (ec, button, "_MouseUp");
 }
 
-static void
-copy (GBEvalContext  *ec,
-      GBObject       *dest_obj,
-      GBObject       *templ_obj)
+/**
+ * cmdbutton_setarg:
+ *   @ec
+ *   @object
+ *   @property
+ *   @val
+ **/
+static gboolean
+cmdbutton_setarg (GBRunEvalContext *ec,
+		  GBRunObject      *object,
+		  int               property,
+		  GBValue          *val)
 {
-	GtkWidget      *w;
-	GBRunCmdButton *dest = PRIV (dest_obj);
+	GBRunCmdButton *button = GBRUN_CMDBUTTON (object);
+	GtkWidget      *bwidget = gbrun_form_item_get_widget (GBRUN_FORM_ITEM (object));
 
-	if (templ_obj)
-		g_warning ("Copy unimplemented");
+	g_return_val_if_fail (button != NULL, FALSE);
+	g_return_val_if_fail (button->label != NULL, FALSE);
+
+	switch (property) {
+	case CAPTION:
+	{
+		char *txt;
+
+		button->real_caption = g_strdup (val->v.s->str);
+
+		txt = gbrun_form_un_shortcutify (button->real_caption, NULL);
+		gtk_label_set_text (button->label, txt);
+		g_free (txt);
+
+		return TRUE;
+	} 
+	case TOOLTIP_TEXT:
+	{
+		GtkTooltips *t = gtk_tooltips_new();
+		gtk_tooltips_set_tip (t, GTK_WIDGET (bwidget),
+		                      val->v.s->str, NULL);
+		return TRUE;
+	}
+	case ENABLED:
+		gtk_widget_set_sensitive (GTK_WIDGET (bwidget),
+					  val->v.bool);
+		return TRUE;
+
+	case DEFAULT:
+		/* FIXME: Is this the right function ? */
+		gtk_widget_set (GTK_WIDGET (bwidget), "can_default", val->v.bool, NULL);
+		g_warning ("FIXME: hacked");
+/*		if (val->v.bool)
+		gtk_window_set_default (button->item.form->window, button->item.widget);*/
+		return TRUE;
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		return FALSE;
+	}
+}
+
+
+/**
+ * cmdbutton_getarg:
+ *   @ec
+ *   @object
+ *   @property
+ **/
+static GBValue *
+cmdbutton_getarg (GBRunEvalContext *ec,
+		  GBRunObject      *object,
+		  int               property)
+{
+	GBRunCmdButton *button = GBRUN_CMDBUTTON (object);
+	GtkWidget      *bwidget = gbrun_form_item_get_widget (
+		GBRUN_FORM_ITEM (object));
+
+	g_return_val_if_fail (button != NULL, NULL);
+	g_return_val_if_fail (button->label != NULL, NULL);
+
+	switch (property) {
+	case CAPTION:
+		return gb_value_new_string_chars (button->real_caption);
+
+	case ENABLED:
+		return gb_value_new_boolean (GTK_WIDGET_SENSITIVE (bwidget));
+
+	case DEFAULT:
+		/* Is this the right macro to use ? */
+		return gb_value_new_boolean (GTK_WIDGET_RECEIVES_DEFAULT (bwidget));
+
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		break;
+	}
+
+	return NULL;
+}
+
+static void
+gbrun_cmdbutton_destroy (GtkObject *object)
+{
+	GBRunCmdButton *cmdbutton = GBRUN_CMDBUTTON (object);
+
+	if (cmdbutton->real_caption)
+		g_free (cmdbutton->real_caption);
+
+	g_warning ("Unimplemented %s destruct", ITEM_NAME);
+}
+
+static void
+gbrun_cmdbutton_construct (GBRunEvalContext *ec, GBRunFormItem *item)
+{
+	GtkWidget *w;
+	GBRunCmdButton *dest = GBRUN_CMDBUTTON (item);
 
 	w = gtk_button_new ();
-	gbrun_form_item_set_widget (dest_obj, w);
+	gbrun_form_item_set_widget (GBRUN_FORM_ITEM (dest), w);
 	dest->label = GTK_LABEL (gtk_label_new ("WIBBLE"));
 	gtk_widget_show (GTK_WIDGET (dest->label));
 
@@ -144,100 +224,56 @@ copy (GBEvalContext  *ec,
 				(GtkSignalFunc) cmdbutton_mousedown, ec);
 	gtk_signal_connect (GTK_OBJECT (w), "released",
 				(GtkSignalFunc) cmdbutton_mouseup, ec);
-
 }
 
 static void
-destruct (GBObject *object)
+gbrun_cmdbutton_class_init (GBRunObjectClass *klass)
 {
-	GBRunCmdButton *button = PRIV (object);
+	GtkObjectClass     *gtk_class    = (GtkObjectClass *) klass;
+	GBRunFormItemClass *form_class   = (GBRunFormItemClass *) klass;
 
-	if (button->real_caption)
-		g_free (button->real_caption);
+	klass->set_arg = cmdbutton_setarg;
+	klass->get_arg = cmdbutton_getarg;
 
-	g_warning ("Unimplemented %s destruct", ITEM_NAME);
+	gbrun_object_add_property (klass, "caption", 
+				   gb_type_string, CAPTION);
+
+	gbrun_object_add_property (klass, "tooltiptext",
+				   gb_type_string, TOOLTIP_TEXT);
+
+        gbrun_object_add_property (klass, "enabled",
+				   gb_type_boolean, ENABLED);
+
+	gbrun_object_add_property (klass, "default",
+				   gb_type_boolean, DEFAULT);
+
+	form_class->construct = gbrun_cmdbutton_construct;
+
+	gtk_class->destroy = gbrun_cmdbutton_destroy;
 }
 
-/**
- * cmdbutton_setarg:
- *   @ec
- *   @object
- *   @property
- *   @val
- **/
-static void
-cmdbutton_setarg (GBRunEvalContext *ec,
-		  GBRunObject      *object,
-		  GBRunObjProperty *property,
-		  GBValue          *val)
+GtkType
+gbrun_cmdbutton_get_type (void)
 {
-	GBRunCmdButton *button = PRIV (object);
-	GtkWidget      *bwidget = gbrun_form_item_get_widget (object);
+	static GtkType object_type = 0;
 
-	g_return_if_fail (button != NULL);
-	g_return_if_fail (button->label != NULL);
+	if (!object_type) {
+		static const GtkTypeInfo object_info = {
+			ITEM_NAME,
+			sizeof (GBRunCmdButton),
+			sizeof (GBRunFormItemClass),
+			(GtkClassInitFunc)  gbrun_cmdbutton_class_init,
+			(GtkObjectInitFunc) NULL,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
+		};
 
-	if      (property == p_caption) {
-		char *txt;
+		object_type = gtk_type_unique (GBRUN_TYPE_FORM_ITEM, &object_info);
+		gtk_type_class (object_type);
+	}
 
-		button->real_caption = g_strdup (val->v.s->str);
-
-		txt = gbrun_form_un_shortcutify (button->real_caption, NULL);
-		gtk_label_set_text (button->label, txt);
-		g_free (txt);
-
-	} else if (property == p_tool_tip_text) {
-		GtkTooltips *t = gtk_tooltips_new();
-		gtk_tooltips_set_tip (t, GTK_WIDGET (bwidget),
-		                      val->v.s->str, NULL);
-
-	} else if (property == p_enabled) {
-		gtk_widget_set_sensitive (GTK_WIDGET (bwidget),
-					  val->v.bool);
-
-	} else if (property == p_default) {
-		/* FIXME: Is this the right function ? */
-		gtk_widget_set (GTK_WIDGET (bwidget), "can_default", val->v.bool, NULL);
-		g_warning ("FIXME: hacked");
-/*		if (val->v.bool)
-		gtk_window_set_default (button->item.form->window, button->item.widget);*/
-
-	} else
-		g_warning ("Unhandled property '%s'", property->name);
-}
-
-
-/**
- * cmdbutton_getarg:
- *   @ec
- *   @object
- *   @property
- **/
-static GBValue *
-cmdbutton_getarg (GBRunEvalContext *ec,
-		  GBRunObject      *object,
-		  GBRunObjProperty *property)
-{
-	GBRunCmdButton *button = PRIV (object);
-	GtkWidget      *bwidget = gbrun_form_item_get_widget (object);
-
-	g_return_val_if_fail (button != NULL, NULL);
-	g_return_val_if_fail (button->label != NULL, NULL);
-
-	if (property == p_caption)
-		return gb_value_new_string_chars (button->real_caption);
-
-	else if (property == p_enabled)
-		return gb_value_new_boolean (GTK_WIDGET_SENSITIVE (bwidget));
-
-	else if (property == p_default)
-		/* Is this the right macro to use ? */
-		return gb_value_new_boolean (GTK_WIDGET_RECEIVES_DEFAULT (bwidget));
-
-	else
-		g_warning ("Unhandled property '%s'", property->name);
-
-	return NULL;
+	return object_type;	
 }
 
 /**
@@ -247,21 +283,7 @@ cmdbutton_getarg (GBRunEvalContext *ec,
 void
 gbrun_cmdbutton_register ()
 {
-	GBRunObjectPrivClass *cmdbutton;
-
-	cmdbutton = gbrun_object_class_get_priv (cmdbutton_class ());
-
-	cmdbutton->set_arg = cmdbutton_setarg;
-	cmdbutton->get_arg = cmdbutton_getarg;
-
-	p_caption       = gbrun_object_add_property_val (cmdbutton, "caption",
-						         GB_VALUE_STRING);
-	p_tool_tip_text = gbrun_object_add_property_val (cmdbutton, "tooltiptext",
-						         GB_VALUE_STRING);
-	p_enabled       = gbrun_object_add_property_val (cmdbutton, "enabled",
-						         GB_VALUE_BOOLEAN);
-	p_default       = gbrun_object_add_property_val (cmdbutton, "default",
-							 GB_VALUE_BOOLEAN);
+	gbrun_cmdbutton_get_type ();
 }
 
 
@@ -272,3 +294,4 @@ void
 gbrun_cmdbutton_shutdown ()
 {
 }
+
