@@ -1,4 +1,3 @@
-
 /*
  * gbrun-form.c
  *
@@ -16,32 +15,7 @@
 #include "gbrun-form.h"
 #include "gbrun-form-item.h"
 
-#define ITEM_NAME "VB.Form"
-
-static void copy     (GBEvalContext *, GBObject *, GBObject *);
-static void destruct (GBObject *);
-
-GBObjectClass *
-gbrun_form_class (void)
-{
-	static GBObjectClass *oc = NULL;
-
-	if (!oc) {
-		static GBRunObjectPrivClass p;
-		gb_object_priv_class_init (&p.priv, ITEM_NAME,
-					   sizeof (GBRunForm),
-					   (GBObjectCopy *)copy,
-					   (GBObjectDestructor *)destruct);
-		gbrun_object_priv_class_init (&p);
-
-		oc = gb_object_class_new_single (&p.priv, gbrun_object_class ());
-		gbrun_object_register (ITEM_NAME, oc);
-	}
-		
-	return oc;
-}
-
-#define PRIV(o) ((GBRunForm *)gb_object_get_priv ((o), gbrun_form_class ()))
+#define ITEM_NAME "gb-vb.form"
 
 /*
  *  temporary.
@@ -53,38 +27,6 @@ delete_event_cb (GtkWidget *window, GdkEventAny *e, gpointer data)
 	return TRUE;
 }
 
-static void
-copy (GBEvalContext  *ec,
-      GBObject       *dest,
-      GBObject       *templ)
-{
-	GBRunForm *form = PRIV (dest);
-
-	if (templ)
-		g_warning ("Copy unimplemented");
-
-	form->window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
-	gtk_signal_connect (GTK_OBJECT (form->window),
-  		            "delete_event",
-		            GTK_SIGNAL_FUNC (delete_event_cb),
-		            dest);
-
-	form->fixed = GTK_FIXED (gtk_fixed_new ()); 
-
-	gtk_container_add (GTK_CONTAINER (form->window),
-			   GTK_WIDGET (form->fixed));
-}
-
-static void
-destruct (GBObject *obj)
-{
-	GBRunForm *form = PRIV (obj);
-
-	if (form->window)
-		gtk_widget_destroy (GTK_WIDGET (form->window));
-	form->window = NULL;
-}
-
 /*
  * _GBEFormFont defines a Microsoft font.
  */
@@ -240,13 +182,16 @@ struct _GBEFormFont {
  *
  */
 
-static GBRunObjProperty *p_caption      = NULL;
-static GBRunObjProperty *p_height       = NULL;
-static GBRunObjProperty *p_left         = NULL;
-static GBRunObjProperty *p_scale_height = NULL;
-static GBRunObjProperty *p_scale_width  = NULL;
-static GBRunObjProperty *p_top          = NULL;
-static GBRunObjProperty *p_width        = NULL;
+enum {
+	ARG_FIRST = 0,
+	CAPTION,
+	HEIGHT,
+	LEFT,
+	SCALE_HEIGHT,
+	SCALE_WIDTH,
+	TOP,
+	WIDTH
+};
 
 /**
  * form_setarg
@@ -259,41 +204,48 @@ static GBRunObjProperty *p_width        = NULL;
  *   This function processes the form properties and calls the appropriate
  * GTK function to set the appropriate properties of a GTK object.
  **/
-static void
+static gboolean
 form_setarg (GBRunEvalContext *ec,
 	     GBRunObject      *object,
-	     GBRunObjProperty *property,
+	     int               property,
 	     GBValue          *val)
 {
-	GBRunForm *form = PRIV (object);
+	GBRunForm *form = GBRUN_FORM (object);
+	GtkWidget *w = GTK_WIDGET (form->window);
 
-	g_return_if_fail (form != NULL);
-	g_return_if_fail (form->window != NULL);
+	g_return_val_if_fail (w != NULL, FALSE);
+	g_return_val_if_fail (form != NULL, FALSE);
 
-	if (property == p_caption)
+	switch (property) {
+	case CAPTION:
 		gtk_window_set_title (form->window, val->v.s->str);
+		return TRUE;
 
-	else if (property == p_height) {
-		GtkWidget *w = GTK_WIDGET (form->window);
+	case HEIGHT:
 		w->requisition.height = GBRUN_FORM_TWIPS_TO_Y (val->v.i);
 		gtk_window_set_default_size (form->window, w->requisition.width,
 				             w->requisition.height);
+		return TRUE;
 
-	} else if (property == p_left) {
-		GtkWidget *w = GTK_WIDGET (form->window);
+	case LEFT:
 		w->allocation.x = GBRUN_FORM_TWIPS_TO_Y (val->v.i);
 		gtk_widget_set_uposition (w, w->allocation.x, w->allocation.y);
+		return TRUE;
 
-	} else if (property == p_top) {
-		GtkWidget *w = GTK_WIDGET (form->window);
+	case TOP:
 		w->allocation.y = GBRUN_FORM_TWIPS_TO_Y (val->v.i);
 		gtk_widget_set_uposition (w, w->allocation.x, w->allocation.y);
+		return TRUE;
 	
-	} else if (property == p_width) {
-		GtkWidget *w = GTK_WIDGET (form->window);
+	case WIDTH:
 		w->requisition.width = GBRUN_FORM_TWIPS_TO_X (val->v.i);
  		gtk_window_set_default_size (form->window, w->requisition.width, 
 		                             w->requisition.height);
+		return TRUE;
+
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		return FALSE;
 	}
 }
 
@@ -308,53 +260,55 @@ form_setarg (GBRunEvalContext *ec,
  **/
 static GBValue *
 form_getarg (GBRunEvalContext *ec,
-	     GBRunObject        *object,
-	     GBRunObjProperty   *property)
+	     GBRunObject      *object,
+	     int               property)
 {
-	GBRunForm *form = PRIV (object);
+	GBRunForm *form = GBRUN_FORM (object);
 
 	g_return_val_if_fail (form != NULL, NULL);
 	g_return_val_if_fail (form->window != NULL, NULL);
-
-	if (property == p_caption)
+	
+	switch (property) {
+	case CAPTION:
 		return gb_value_new_string_chars (form->window->title);
 
-	else if (property == p_width) {
+	case WIDTH: {
 		guint i = GTK_WIDGET (form->window)->allocation.width;
 		return gb_value_new_int (GBRUN_FORM_X_TO_TWIPS (i));
+	}
 
-	} else if (property == p_height) {
+	case HEIGHT: {
 		guint i = GTK_WIDGET (form->window)->allocation.height;
 		return gb_value_new_int (GBRUN_FORM_Y_TO_TWIPS (i));
 	}
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		break;
+	}
 
 	return NULL;
 }
 
 static GBValue *
 form_show (GBRunEvalContext *ec,
-	   GBRunObject      *object,
+	   GBRunObject      *form,
 	   GBValue         **args)
 {
-	GBRunForm *form = PRIV (object);
-
 	/* FIXME: loads if not already loaded !? */
 	if (args [0] || args [1])
 		g_warning ("Modality & owner unimplemented");
 
-	gtk_widget_show (GTK_WIDGET (form->window));
+	gtk_widget_show (GTK_WIDGET (GBRUN_FORM (form)->window));
 
 	return gb_value_new_empty ();
 }
 
 static GBValue *
 form_hide (GBRunEvalContext *ec,
-	   GBRunObject      *object,
+	   GBRunObject      *form,
 	   GBValue         **args)
 {
-	GBRunForm *form = PRIV (object);
-
-	gtk_widget_hide (GTK_WIDGET (form->window));
+	gtk_widget_hide (GTK_WIDGET (GBRUN_FORM (form)->window));
 
 	return gb_value_new_empty ();
 }
@@ -365,25 +319,7 @@ form_hide (GBRunEvalContext *ec,
 void
 gbrun_form_register (void)
 {
-	GBRunObjectPrivClass *klass;
-	
-	klass = gbrun_object_class_get_priv (gbrun_form_class ());
-
-	klass->set_arg = form_setarg;
-	klass->get_arg = form_getarg;
-
-	p_caption      = gbrun_object_add_property_val (klass, "caption",      GB_VALUE_STRING);
-	p_height       = gbrun_object_add_property_val (klass, "clientheight", GB_VALUE_INT);
-	p_left         = gbrun_object_add_property_val (klass, "clientleft",   GB_VALUE_INT);
-	p_scale_height = gbrun_object_add_property_val (klass, "scaleheight",  GB_VALUE_INT);
-	p_scale_width  = gbrun_object_add_property_val (klass, "scalewidth",   GB_VALUE_INT);
-	p_top          = gbrun_object_add_property_val (klass, "clienttop",    GB_VALUE_INT);
-	p_width        = gbrun_object_add_property_val (klass, "clientwidth",  GB_VALUE_INT);
-
-	gbrun_object_add_method_arg (klass, "sub;show;modal,integer,byval,0;ownerform,variant,byref,0;g",
-				     form_show);
-	gbrun_object_add_method_arg (klass, "sub;hide;g",
-				     form_hide);
+	gbrun_form_get_type ();
 }
 
 
@@ -408,21 +344,26 @@ gbrun_form_shutdown (void)
  * into gbrun_object_set_arg.
  **/
 static void
-gbrun_form_pass_properties (GBRunEvalContext *ec, GBRunObject *obj,
-			    GBFormItem *item)
+gbrun_form_pass_properties (GBRunEvalContext *ec,
+			    GBRunObject      *obj,
+			    GBFormItem       *item)
 {
 	GSList *l;
 	
 	for (l = item->properties; l; l = l->next) {
 		GBFormProperty *prop = l->data;
+		GBObjRef ref;
+		
+		ref.method = FALSE;
+		ref.name   = prop->name;
+		ref.parms  = NULL;
 
-		if (gbrun_object_get_property (GBRUN_OBJECT_GET_CLASS (obj),
-					       prop->name))
-			gbrun_object_set_arg (ec, obj,
-					      prop->name, prop->value);
+		if (gbrun_object_has_property (GBRUN_OBJECT_GET_CLASS (obj), prop->name))
+			gb_object_assign (GB_EVAL_CONTEXT (ec), GB_OBJECT (obj),
+					  &ref, prop->value, FALSE);
 		else
 			g_warning ("Missing property '%s' on '%s' named '%s'", 
-			           prop->name, gbrun_object_name (obj),
+			           prop->name, gbrun_object_name (GBRUN_OBJECT (obj)),
 				   item->name);
 	}
 }
@@ -433,69 +374,58 @@ gbrun_form_pass_properties (GBRunEvalContext *ec, GBRunObject *obj,
  *   @obj: 
  **/
 void
-gbrun_form_show (GBRunObject *obj)
+gbrun_form_show (GBRunForm *form)
 {
-	GBRunForm *form;
-
-	g_return_if_fail (obj != NULL);
-
-	form = PRIV (obj);
-	g_return_if_fail (form != NULL);
+	g_return_if_fail (GBRUN_IS_FORM (form));
 	
 	gtk_widget_show_all (GTK_WIDGET (form->window));
 }
 
 static void
-gbrun_form_add (GBObject          *form,
-		GBObject          *obj,
+gbrun_form_add (GBRunForm         *form,
+		GBRunFormItem     *item,
 		const char        *name,
 		const GBParseData *module)
 {
-	GBRunFormItem *fi;
-
-	g_return_if_fail (obj != NULL);
+	g_return_if_fail (item != NULL);
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (form != NULL);
 
-	fi = gb_object_get_priv (obj, gbrun_form_item_class ());
-	g_return_if_fail (fi != NULL);
+	item->form = form;
+	item->name = g_strdup (name);
 
-	fi->form = form;
-	fi->name = g_strdup (name);
-
-	gtk_fixed_put (PRIV (form)->fixed, fi->widget, 0, 0);
+	gtk_fixed_put (form->fixed, item->widget, 0, 0);
 }
 
 gboolean
 gbrun_form_invoke (GBRunEvalContext *ec,
-		   GBRunObject      *form,
+		   GBRunForm        *form,
 		   const char       *method)
 {
-	gboolean ret = TRUE;
-
-	g_return_val_if_fail (form != NULL, FALSE);
+	GBValue  *ignore;
+	gboolean  ret;
 	
-	if (gbrun_object_get_method (GBRUN_OBJECT_GET_CLASS (form),
-				     method)) {
-		GBObjRef  ref;
-		GBValue  *ignore;
-		
-		ref.method = FALSE;
+	g_return_val_if_fail (form != NULL, FALSE);
+
+	if (gbrun_object_has_method (GBRUN_OBJECT_GET_CLASS (form), method)) {
+		GBObjRef ref;
+
+		ref.method = TRUE;
 		ref.name   = method;
 		ref.parms  = NULL;
 
-		ignore = gbrun_method_invoke (ec, form, &ref);
-
-		if (ignore)
+		if ((ignore = gbrun_objref_deref (ec, GB_OBJECT (form),
+						  &ref, TRUE))) {
 			gb_value_destroy (ignore);
-		else {
-			g_warning ("Error invoking '%s' : '%s", method,
-				   gb_eval_context_get_text (GB_EVAL_CONTEXT (ec)));
-			ret = FALSE;
+			ret = TRUE;
+		} else {
+			if (gb_eval_exception (GB_EVAL_CONTEXT (ec))) {
+				g_warning ("Error invoking '%s' : '%s", method,
+					   gb_eval_context_get_text (GB_EVAL_CONTEXT (ec)));
+				gb_eval_context_reset (GB_EVAL_CONTEXT (ec));
+				ret = FALSE;
+			}
 		}
-	} else {
-		g_warning ("No '%s' handler on form", method);
-		ret = FALSE;
 	}
 
 	return ret;
@@ -508,48 +438,47 @@ gbrun_form_invoke (GBRunEvalContext *ec,
  * 
  **/
 void
-gbrun_form_init (GBRunEvalContext *ec, GBRunObject *obj,
+gbrun_form_init (GBRunEvalContext  *ec,
+		 GBRunForm         *form,
 		 const GBParseData *pd)
 {
 	GSList        *l;
-	GBRunForm     *form;
 	GBFormItem    *item;
 
 	g_return_if_fail (ec != NULL);
 	g_return_if_fail (pd != NULL);
-	g_return_if_fail (obj != NULL);
-
-	form = PRIV (obj);
 	g_return_if_fail (form != NULL);
 
 	item = pd->form;
-	gbrun_form_pass_properties (ec, obj, item);
+	gbrun_form_pass_properties (ec, GBRUN_OBJECT (form), item);
 
 	for (l = item->children; l; l = l->next) {
 		GBFormItem       *i = l->data;
-		GBRunObjectClass *klass;
-		GBRunObject      *object;
+		GtkType           type;
+		GBRunFormItem    *item;
 
-		klass = gbrun_object_lookup (i->type);
-		if (!klass) {
+		type = gb_gtk_type_from_name (i->type);
+		if (!type) {
 			g_warning ("Unknown sub-form type '%s'", i->type);
 			continue;
 		}
 
-		object = gbrun_object_new (ec, i->type);
-		if (gb_object_implements (object, gbrun_form_item_class ()))
-			gbrun_form_add (obj, object, i->name, pd);
+		item = gbrun_form_item_new (ec, type);
+
+		if (GBRUN_IS_FORM_ITEM (item))
+			gbrun_form_add (form, item, i->name, pd);
 		else
 			g_warning ("Non form item '%s' put into form", i->type);
 		
-		gbrun_form_pass_properties (ec, object, i);
+		gbrun_form_pass_properties (ec, GBRUN_OBJECT (item), i);
 		
 		/* Add it to the stack */
-		gbrun_object_ref (ec, object);
-		gbrun_stack_add  (ec, i->name, gb_value_new_object (object),
+		gbrun_object_ref (ec, item);
+		gbrun_stack_add  (ec, i->name,
+				  gb_value_new_object (GB_OBJECT (item)),
 				  GBRUN_STACK_MODULE);
 	}
-	gbrun_form_show (obj);
+	gbrun_form_show (form);
 }
 
 void
@@ -639,3 +568,78 @@ gbrun_form_un_shortcutify (const char *txt, char *shortcut)
 
 	return ans;
 }
+
+
+static void
+gbrun_form_destroy (GtkObject *object)
+{
+	GBRunForm *form = GBRUN_FORM (object);
+
+	if (form->window)
+		gtk_widget_destroy (GTK_WIDGET (form->window));
+	form->window = NULL;
+}
+
+static void
+gbrun_form_instance_init (GBRunForm *form)
+{
+	form->window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+	gtk_signal_connect (GTK_OBJECT (form->window),
+  		            "delete_event",
+		            GTK_SIGNAL_FUNC (delete_event_cb),
+		            form);
+
+	form->fixed = GTK_FIXED (gtk_fixed_new ()); 
+
+	gtk_container_add (GTK_CONTAINER (form->window),
+			   GTK_WIDGET (form->fixed));
+}
+
+static void
+gbrun_form_class_init (GBRunObjectClass *klass)
+{
+	GtkObjectClass *gtk_class = (GtkObjectClass *) klass;
+
+	klass->set_arg = form_setarg;
+	klass->get_arg = form_getarg;
+
+	gbrun_object_add_property (klass, "caption",	  gb_type_string, CAPTION);
+	gbrun_object_add_property (klass, "clientheight", gb_type_int, HEIGHT);
+	gbrun_object_add_property (klass, "clientleft",   gb_type_int, LEFT);
+	gbrun_object_add_property (klass, "scaleheight",  gb_type_int, SCALE_HEIGHT);
+	gbrun_object_add_property (klass, "scalewidth",   gb_type_int, SCALE_WIDTH);
+	gbrun_object_add_property (klass, "clienttop",    gb_type_int, TOP);
+	gbrun_object_add_property (klass, "clientwidth",  gb_type_int, WIDTH);
+
+	gbrun_object_add_method_arg (
+		klass, "sub;show;modal,integer,byval,0;ownerform,variant,byref,0;g", form_show);
+
+	gbrun_object_add_method_arg (
+		klass, "sub;hide;g", form_hide);
+
+	gtk_class->destroy = gbrun_form_destroy;
+}
+
+GtkType
+gbrun_form_get_type (void)
+{
+	static GtkType object_type = 0;
+
+	if (!object_type) {
+		static const GtkTypeInfo object_info = {
+			ITEM_NAME,
+			sizeof (GBRunForm),
+			sizeof (GBRunFormClass),
+			(GtkClassInitFunc)  gbrun_form_class_init,
+			(GtkObjectInitFunc) gbrun_form_instance_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
+		};
+
+		object_type = gtk_type_unique (GBRUN_TYPE_OBJECT, &object_info);
+		gtk_type_class (object_type);
+	}
+
+	return object_type;	
+}
