@@ -8,41 +8,24 @@
  */
 #include "gbrun-form-item.h"
 
-#define ITEM_NAME "VB.TextBox"
+#define ITEM_NAME "gb-vb.textbox"
 
-struct _GBRunTextBox {
-	GBBoolean	locked;
+
+enum {
+	ARG_FIRST = 0,
+	LOCKED,
+	TOOLTIP_TEXT,
+	TEXT,
+	ALIGNMENT
 };
 
-static void copy (GBEvalContext *, GBObject *, GBObject *);
+#define GBRUN_TEXTBOX(obj) (GTK_CHECK_CAST ((obj), gbrun_textbox_get_type (), GBRunTextBox))
 
-static void
-destruct (GBObject *object)
-{
-	g_warning ("Unimplemented %s destruct", ITEM_NAME);
-}
+typedef struct {
+	GBRunFormItem   item;
 
-static GBObjectClass *
-textbox_class (void)
-{
-	static GBObjectClass *oc = NULL;
-
-	if (!oc) {
-		static GBRunObjectPrivClass p;
-		gb_object_priv_class_init (&p.priv, ITEM_NAME,
-					   sizeof (GBRunTextBox),
-					   (GBObjectCopy *)copy,
-					   (GBObjectDestructor *)destruct);
-		gbrun_object_priv_class_init (&p);
-
-		oc = gb_object_class_new_single (&p.priv, gbrun_form_item_class ());
-		gbrun_object_register (ITEM_NAME, oc);
-	}
-		
-	return oc;
-}
-
-#define PRIV(o) (gb_object_get_priv ((o), textbox_class ()))
+	gboolean        locked;
+} GBRunTextBox;
 
 /*
  * _GBETextBox defines a VB Text Box
@@ -114,50 +97,57 @@ struct _GBETextBox {
 };
 
 
-static GBRunObjProperty *p_locked        = NULL;
-static GBRunObjProperty *p_tool_tip_text = NULL;
-static GBRunObjProperty *p_text          = NULL;
-static GBRunObjProperty *p_alignment     = NULL;
 
 /**
  * textbox_setarg
  *
  **/
-static void
+static gboolean
 textbox_setarg (GBRunEvalContext *ec,
-	        GBRunObject      *object,
-	        GBRunObjProperty *property,
-	        GBValue          *val)
+		GBRunObject      *object,
+		int               property,
+		GBValue          *val)
 {
-	GBRunTextBox *textbox = PRIV (object);
-	GtkEntry     *entry = GTK_ENTRY (gbrun_form_item_get_widget (object));
+	GBRunTextBox *textbox = GBRUN_TEXTBOX (object);
+	GtkEntry     *entry = GTK_ENTRY (
+		gbrun_form_item_get_widget (GBRUN_FORM_ITEM (object)));
 
-	g_return_if_fail (textbox != NULL);
+	g_return_val_if_fail (textbox != NULL, FALSE);
 
-	if (property == p_locked) {
+	switch (property) {
+	case LOCKED:
 		textbox->locked = (val->v.i == VB_FALSE);
 		gtk_entry_set_editable (entry, 
 		                        textbox->locked);
+		return TRUE;
 
-	} else if (property == p_tool_tip_text) {
+	case TOOLTIP_TEXT: {
 		GtkTooltips *t = gtk_tooltips_new();
 		gtk_tooltips_set_tip (t, GTK_WIDGET (entry), 
 		                      val->v.s->str, NULL);
-		
-	} else if (property == p_text) {
+		return TRUE;
+	}
+
+	case TEXT: 
 		gtk_entry_set_editable (entry, TRUE);
 		gtk_entry_set_text (entry, val->v.s->str);
 		gtk_entry_set_editable (entry, textbox->locked);
+		return TRUE;
 
-	} else if (property == p_alignment) {
+	case ALIGNMENT: {
 		static gboolean warned = FALSE;
 		if (!warned) {
 			g_warning ("GtkEntry alignment unsupported");
 			warned = TRUE;
 		}
 
-	} else
-		g_warning ("Unhandled property '%s'", property->name);
+		return TRUE;
+	}
+
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		return FALSE;
+	}
 }
 
 
@@ -169,39 +159,83 @@ textbox_setarg (GBRunEvalContext *ec,
  **/
 static GBValue *
 textbox_getarg (GBRunEvalContext *ec,
-	        GBRunObject      *object,
-	        GBRunObjProperty *property)
+		GBRunObject      *object,
+		int               property)
 {
-	GBRunTextBox *textbox = PRIV (object);
-	GtkEntry     *entry = GTK_ENTRY (gbrun_form_item_get_widget (object));
+	GBRunTextBox *textbox = GBRUN_TEXTBOX (object);
+	GtkEntry     *entry = GTK_ENTRY (
+		gbrun_form_item_get_widget (GBRUN_FORM_ITEM (object)));
 
 	g_return_val_if_fail (textbox != NULL, NULL);
 
-	if (property == p_locked)
+	switch (property) {
+	case LOCKED:
 		return gb_value_new_boolean (textbox->locked);
 
-	else if (property == p_text)
-		return gb_value_new_string_chars (gtk_entry_get_text (entry));
+	case TEXT:
+		return gb_value_new_string_chars (
+			gtk_entry_get_text (entry));
 
-	else
-		g_warning ("Unhandled property '%s'", property->name);
-
-	return NULL;
+	default:
+		g_warning ("Unhandled property '%d'", property);
+		return NULL;
+	}
 }
 
+static void
+gbrun_textbox_construct (GBRunEvalContext *ec, GBRunFormItem *item)
+{
+	GBRunTextBox *dest = GBRUN_TEXTBOX (item);
+
+	gbrun_form_item_set_widget (item, gtk_entry_new ());
+	dest->locked = FALSE;
+}
 
 static void
-copy (GBEvalContext  *ec,
-      GBObject       *dest_obj,
-      GBObject       *templ_obj)
+gbrun_textbox_class_init (GBRunObjectClass *klass)
 {
-	GBRunTextBox *dest = PRIV (dest_obj);
+	GBRunFormItemClass *form_class   = (GBRunFormItemClass *) klass;
 
-	if (templ_obj)
-		g_warning ("Copy unimplemented");
+	klass->set_arg = textbox_setarg;
+	klass->get_arg = textbox_getarg;
 
-	gbrun_form_item_set_widget (dest_obj, gtk_entry_new ());
-	dest->locked = FALSE;
+	gbrun_object_add_property (klass, "locked", 
+				   gb_type_int, LOCKED);
+
+	gbrun_object_add_property (klass, "tooltiptext",
+				   gb_type_string, TOOLTIP_TEXT);
+
+        gbrun_object_add_property (klass, "text",
+				   gb_type_string, TEXT);
+
+	gbrun_object_add_property (klass, "alignment",
+				   gb_type_int, ALIGNMENT);
+
+	form_class->construct = gbrun_textbox_construct;
+}
+
+GtkType
+gbrun_textbox_get_type (void)
+{
+	static GtkType object_type = 0;
+
+	if (!object_type) {
+		static const GtkTypeInfo object_info = {
+			ITEM_NAME,
+			sizeof (GBRunTextBox),
+			sizeof (GBRunFormItemClass),
+			(GtkClassInitFunc)  gbrun_textbox_class_init,
+			(GtkObjectInitFunc) NULL,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
+		};
+
+		object_type = gtk_type_unique (GBRUN_TYPE_FORM_ITEM, &object_info);
+		gtk_type_class (object_type);
+	}
+
+	return object_type;	
 }
 
 /**
@@ -211,21 +245,7 @@ copy (GBEvalContext  *ec,
 void
 gbrun_textbox_register ()
 {
-	GBRunObjectPrivClass *textbox;
-	
-	textbox = gbrun_object_class_get_priv (textbox_class ());
-
-	textbox->set_arg = textbox_setarg;
-	textbox->get_arg = textbox_getarg;
-	
-	p_locked        = gbrun_object_add_property_val (textbox, "locked", 
-	                                                 GB_VALUE_INT);
-	p_tool_tip_text = gbrun_object_add_property_val (textbox, "tooltiptext",
-						         GB_VALUE_STRING);
-	p_text          = gbrun_object_add_property_val (textbox, "Text",
-							 GB_VALUE_STRING);
-	p_alignment     = gbrun_object_add_property_val (textbox, "Alignment",
-							 GB_VALUE_INT);
+	gbrun_textbox_get_type ();	
 }
 
 
